@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008-2009 by Andrzej Rybczak                            *
+ *   Copyright (C) 2008-2010 by Andrzej Rybczak                            *
  *   electricityispower@gmail.com                                          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -36,8 +36,14 @@
 # include "tag_editor.h"
 #endif // HAVE_TAGLIB_H
 
-using namespace Global;
-using namespace MPD;
+using Global::MainHeight;
+using Global::MainStartY;
+using Global::myScreen;
+using Global::RedrawHeader;
+
+using MPD::itDirectory;
+using MPD::itSong;
+using MPD::itPlaylist;
 
 Browser *myBrowser = new Browser;
 
@@ -51,10 +57,10 @@ const char *Browser::SupportedExtensions[] =
 
 void Browser::Init()
 {
-	w = new Menu<Item>(0, MainStartY, COLS, MainHeight, Config.columns_in_browser ? Display::Columns() : "", Config.main_color, brNone);
+	w = new Menu<MPD::Item>(0, MainStartY, COLS, MainHeight, Config.columns_in_browser ? Display::Columns() : "", Config.main_color, brNone);
 	w->HighlightColor(Config.main_highlight_color);
-	w->SetTimeout(ncmpcpp_window_timeout);
 	w->CyclicScrolling(Config.use_cyclic_scrolling);
+	w->CenteredCursor(Config.centered_cursor);
 	w->SetSelectPrefix(&Config.selected_item_prefix);
 	w->SetSelectSuffix(&Config.selected_item_suffix);
 	w->SetItemDisplayer(Display::Items);
@@ -72,7 +78,11 @@ void Browser::Resize()
 void Browser::SwitchTo()
 {
 	if (myScreen == this)
-		return;
+	{
+#		ifndef WIN32
+		myBrowser->ChangeBrowseMode();
+#		endif // !WIN32
+	}
 	
 	if (!isInitialized)
 		Init();
@@ -80,7 +90,13 @@ void Browser::SwitchTo()
 	if (hasToBeResized)
 		Resize();
 	
+	if (isLocal()) // local browser doesn't support sorting by mtime
+		Config.browser_sort_by_mtime = 0;
+	
 	w->Empty() ? myBrowser->GetDirectory(itsBrowsedDir) : myBrowser->UpdateItemList();
+
+	if (myScreen != this && myScreen->isTabbable())
+		Global::myPrevScreen = myScreen;
 	myScreen = this;
 	RedrawHeader = 1;
 }
@@ -88,7 +104,7 @@ void Browser::SwitchTo()
 std::basic_string<my_char_t> Browser::Title()
 {
 	std::basic_string<my_char_t> result = U("Browse: ");
-	result += Scroller(TO_WSTRING(itsBrowsedDir), itsScrollBeginning, w->GetWidth()-result.length()-(Config.new_design ? 2 : VolumeState.length()));
+	result += Scroller(TO_WSTRING(itsBrowsedDir), itsScrollBeginning, w->GetWidth()-result.length()-(Config.new_design ? 2 : Global::VolumeState.length()));
 	return result;
 }
 
@@ -97,7 +113,7 @@ void Browser::EnterPressed()
 	if (w->Empty())
 		return;
 	
-	const Item &item = w->Current();
+	const MPD::Item &item = w->Current();
 	switch (item.type)
 	{
 		case itDirectory:
@@ -113,7 +129,7 @@ void Browser::EnterPressed()
 		}
 		case itPlaylist:
 		{
-			SongList list;
+			MPD::SongList list;
 			Mpd.GetPlaylistContent(locale_to_utf_cpy(item.name), list);
 			if (myPlaylist->Add(list, 1))
 				ShowMessage("Loading and playing playlist %s...", item.name.c_str());
@@ -138,7 +154,7 @@ void Browser::SpacePressed()
 	if (itsBrowsedDir != "/" && w->Choice() == 0 /* parent dir */)
 		return;
 	
-	const Item &item = w->Current();
+	const MPD::Item &item = w->Current();
 	switch (item.type)
 	{
 		case itDirectory:
@@ -146,11 +162,11 @@ void Browser::SpacePressed()
 			if (itsBrowsedDir != "/" && !w->Choice())
 				break; // do not let add parent dir.
 			
-			SongList list;
+			MPD::SongList list;
 #			ifndef WIN32
 			if (isLocal())
 			{
-				ItemList items;
+				MPD::ItemList items;
 				ShowMessage("Scanning \"%s\"...", item.name.c_str());
 				myBrowser->GetLocalDirectory(items, item.name, 1);
 				list.reserve(items.size());
@@ -174,7 +190,7 @@ void Browser::SpacePressed()
 		}
 		case itPlaylist:
 		{
-			SongList list;
+			MPD::SongList list;
 			Mpd.GetPlaylistContent(locale_to_utf_cpy(item.name), list);
 			if (myPlaylist->Add(list, 0))
 				ShowMessage("Loading playlist %s...", item.name.c_str());
@@ -240,19 +256,31 @@ void Browser::GetSelectedSongs(MPD::SongList &v)
 {
 	std::vector<size_t> selected;
 	w->GetSelected(selected);
+	if (selected.empty())
+		selected.push_back(w->Choice());
 	for (std::vector<size_t>::const_iterator it = selected.begin(); it != selected.end(); ++it)
 	{
-		const Item &item = w->at(*it);
+		const MPD::Item &item = w->at(*it);
 		switch (item.type)
 		{
 			case itDirectory:
 			{
-				Mpd.GetDirectoryRecursive(locale_to_utf_cpy(item.name), v);
+#				ifndef WIN32
+				if (isLocal())
+				{
+					MPD::ItemList list;
+					GetLocalDirectory(list, item.name, 1);
+					for (MPD::ItemList::const_iterator j = list.begin(); j != list.end(); ++j)
+						v.push_back(j->song);
+				}
+				else
+#				endif // !WIN32
+					Mpd.GetDirectoryRecursive(locale_to_utf_cpy(item.name), v);
 				break;
 			}
 			case itSong:
 			{
-				v.push_back(new Song(*item.song));
+				v.push_back(new MPD::Song(*item.song));
 				break;
 			}
 			case itPlaylist:
@@ -291,7 +319,8 @@ void Browser::LocateSong(const MPD::Song &s)
 	
 	itsBrowseLocally = !s.isFromDB();
 	
-	SwitchTo();
+	if (myScreen != this)
+		SwitchTo();
 	
 	std::string option = s.toString(Config.song_status_format);
 	locale_to_utf(option);
@@ -324,28 +353,29 @@ void Browser::GetDirectory(std::string dir, std::string subdir)
 		if (w->at(i).type == itSong)
 			delete w->at(i).song;
 	
-	w->Clear(0);
+	w->Clear();
 	
 	if (dir != "/")
 	{
-		Item parent;
+		MPD::Item parent;
 		size_t slash = dir.rfind("/");
-		parent.song = reinterpret_cast<Song *>(1); // in that way we assume that's really parent dir
+		parent.song = reinterpret_cast<MPD::Song *>(1); // in that way we assume that's really parent dir
 		parent.name = slash != std::string::npos ? dir.substr(0, slash) : "/";
 		parent.type = itDirectory;
 		utf_to_locale(parent.name);
 		w->AddOption(parent);
 	}
 	
-	ItemList list;
+	MPD::ItemList list;
 #	ifndef WIN32
 	isLocal() ? GetLocalDirectory(list) : Mpd.GetDirectory(dir, list);
 #	else
 	Mpd.GetDirectory(dir, list);
 #	endif // !WIN32
-	sort(list.begin(), list.end(), CaseInsensitiveSorting());
+	if (!isLocal()) // local directory is already sorted
+		sort(list.begin(), list.end(), CaseInsensitiveSorting());
 	
-	for (ItemList::iterator it = list.begin(); it != list.end(); ++it)
+	for (MPD::ItemList::iterator it = list.begin(); it != list.end(); ++it)
 	{
 		switch (it->type)
 		{
@@ -384,7 +414,7 @@ void Browser::GetDirectory(std::string dir, std::string subdir)
 }
 
 #ifndef WIN32
-void Browser::GetLocalDirectory(ItemList &v, const std::string &directory, bool recursively) const
+void Browser::GetLocalDirectory(MPD::ItemList &v, const std::string &directory, bool recursively) const
 {
 	DIR *dir = opendir((directory.empty() ? itsBrowsedDir : directory).c_str());
 	
@@ -407,11 +437,12 @@ void Browser::GetLocalDirectory(ItemList &v, const std::string &directory, bool
 		}
 	}
 	
+	size_t old_size = v.size();
 	while ((file = readdir(dir)))
 	{
 		if (!Config.local_browser_show_hidden_files && file->d_name[0] == '.')
 			continue;
-		Item new_item;
+		MPD::Item new_item;
 		full_path = directory.empty() ? itsBrowsedDir : directory;
 		if (itsBrowsedDir != "/")
 			full_path += "/";
@@ -420,7 +451,10 @@ void Browser::GetLocalDirectory(ItemList &v, const std::string &directory, bool
 		if (S_ISDIR(file_stat.st_mode))
 		{
 			if (recursively)
+			{
 				GetLocalDirectory(v, full_path, 1);
+				old_size = v.size();
+			}
 			else
 			{
 				new_item.type = itDirectory;
@@ -431,17 +465,17 @@ void Browser::GetLocalDirectory(ItemList &v, const std::string &directory, bool
 		else if (hasSupportedExtension(file->d_name))
 		{
 			new_item.type = itSong;
-			mpd_Song *s = mpd_newSong();
-			s->file = str_pool_get(full_path.c_str());
+			mpd_pair file_pair = { "file", full_path.c_str() };
+			new_item.song = new MPD::Song(mpd_song_begin(&file_pair));
 #			ifdef HAVE_TAGLIB_H
 			if (!recursively)
-				TagEditor::ReadTags(s);
+				TagEditor::ReadTags(*new_item.song);
 #			endif // HAVE_TAGLIB_H
-			new_item.song = new Song(s);
 			v.push_back(new_item);
 		}
 	}
 	closedir(dir);
+	std::sort(v.begin()+old_size, v.end(), CaseInsensitiveSorting());
 }
 
 void Browser::ClearDirectory(const std::string &path) const
@@ -475,9 +509,15 @@ void Browser::ClearDirectory(const std::string &path) const
 		if (S_ISDIR(file_stat.st_mode))
 			ClearDirectory(full_path);
 		if (remove(full_path.c_str()) == 0)
-			ShowMessage("Deleting \"%s\"...", full_path.c_str());
+		{
+			static const char msg[] = "Deleting \"%s\"...";
+			ShowMessage(msg, Shorten(TO_WSTRING(full_path), COLS-static_strlen(msg)).c_str());
+		}
 		else
-			ShowMessage("Couldn't remove \"%s\": %s", full_path.c_str(), strerror(errno));
+		{
+			static const char msg[] = "Couldn't remove \"%s\": %s";
+			ShowMessage(msg, Shorten(TO_WSTRING(full_path), COLS-static_strlen(msg)-25).c_str(), strerror(errno));
+		}
 	}
 	closedir(dir);
 }
