@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008-2009 by Andrzej Rybczak                            *
+ *   Copyright (C) 2008-2010 by Andrzej Rybczak                            *
  *   electricityispower@gmail.com                                          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -30,32 +30,38 @@
 #include <stdexcept>
 
 #include "settings.h"
-#include "str_pool.h"
 
 namespace
 {
-	inline bool char_non_ascii(char ch)
+	bool is_utf8(const char *s)
 	{
-		return (ch & 0x80) != 0;
+		for (; *s; ++s)
+		{
+			if (*s & 0x80) // 1xxxxxxx
+			{
+				char c = 0x40;
+				unsigned i = 0;
+				while (c & *s)
+					++i, c >>= 1;
+				if (i < 1 || i > 3) // not 110xxxxx, 1110xxxx, 11110xxx
+					return false;
+				for (unsigned j = 0; j < i; ++j)
+					if (!*++s || !(*s & 0x80) || *s & 0x40) // 10xxxxxx
+						return false;
+			}
+		}
+		return true;
 	}
 	
 	bool has_non_ascii_chars(const char *s)
 	{
 		for (; *s; ++s)
-			if (char_non_ascii(*s))
+			if (*s & 0x80)
 				return true;
 		return false;
 	}
 	
-	bool has_non_ascii_chars(const std::string &s)
-	{
-		for (std::string::const_iterator it = s.begin(); it != s.end(); ++it)
-			if (char_non_ascii(*it))
-				return true;
-		return false;
-	}
-	
-	void charset_convert(const char *from, const char *to, const char *&inbuf, size_t len = 0)
+	void charset_convert(const char *from, const char *to, const char *&inbuf, bool delete_old, size_t len = 0)
 	{
 		if (!inbuf || !from || !to)
 			return;
@@ -81,20 +87,20 @@ namespace
 		}
 		iconv_close(cd);
 		*outbuf = 0;
-		str_pool_put(instart);
-		inbuf = str_pool_get(outstart);
-		delete [] outstart;
+		if (delete_old)
+			delete [] instart;
+		inbuf = outstart;
 	}
 }
 
 void utf_to_locale(std::string &s)
 {
-	if (s.empty() || Config.system_encoding.empty() || !has_non_ascii_chars(s))
+	if (s.empty() || Config.system_encoding.empty() || !has_non_ascii_chars(s.c_str()))
 		return;
-	const char *tmp = str_pool_get(s.c_str());
-	charset_convert("utf-8", Config.system_encoding.c_str(), tmp, s.length());
+	const char *tmp = strdup(s.c_str());
+	charset_convert("utf-8", Config.system_encoding.c_str(), tmp, 1, s.length());
 	s = tmp;
-	str_pool_put(tmp);
+	delete [] tmp;
 }
 
 std::string utf_to_locale_cpy(const std::string &s)
@@ -106,12 +112,12 @@ std::string utf_to_locale_cpy(const std::string &s)
 
 void locale_to_utf(std::string &s)
 {
-	if (s.empty() || Config.system_encoding.empty() || !has_non_ascii_chars(s))
+	if (s.empty() || Config.system_encoding.empty() || !has_non_ascii_chars(s.c_str()) || is_utf8(s.c_str()))
 		return;
-	const char *tmp = str_pool_get(s.c_str());
-	charset_convert(Config.system_encoding.c_str(), "utf-8", tmp, s.length());
+	const char *tmp = strdup(s.c_str());
+	charset_convert(Config.system_encoding.c_str(), "utf-8", tmp, 1, s.length());
 	s = tmp;
-	str_pool_put(tmp);
+	delete [] tmp;
 }
 
 std::string locale_to_utf_cpy(const std::string &s)
@@ -121,18 +127,18 @@ std::string locale_to_utf_cpy(const std::string &s)
 	return result;
 }
 
-void str_pool_utf_to_locale(const char *&s)
+void utf_to_locale(const char *&s, bool delete_old)
 {
 	if (!s || Config.system_encoding.empty() || !has_non_ascii_chars(s))
 		return;
-	charset_convert("utf-8", Config.system_encoding.c_str(), s);
+	charset_convert("utf-8", Config.system_encoding.c_str(), s, delete_old);
 }
 
-void str_pool_locale_to_utf(const char *&s)
+void locale_to_utf(const char *&s, bool delete_old)
 {
-	if (!s || Config.system_encoding.empty() || !has_non_ascii_chars(s))
+	if (!s || Config.system_encoding.empty() || !has_non_ascii_chars(s) || is_utf8(s))
 		return;
-	charset_convert(Config.system_encoding.c_str(), "utf-8", s);
+	charset_convert(Config.system_encoding.c_str(), "utf-8", s, delete_old);
 }
 
 #endif // HAVE_ICONV_H
