@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008-2009 by Andrzej Rybczak                            *
+ *   Copyright (C) 2008-2010 by Andrzej Rybczak                            *
  *   electricityispower@gmail.com                                          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -20,6 +20,7 @@
 
 #include "display.h"
 #include "helpers.h"
+#include "info.h"
 #include "playlist.h"
 
 std::string Display::Columns()
@@ -38,7 +39,12 @@ std::string Display::Columns()
 	
 	for (std::vector<Column>::const_iterator it = Config.columns.begin(); it != Config.columns.end(); ++it)
 	{
-		width = last_fixed && it == next2last ? COLS-where-(++next2last)->width : (it->width*(it->fixed ? 1 : COLS/100.0));
+		if (it == Config.columns.end()-1)
+			width = COLS-where;
+		else if (last_fixed && it == next2last)
+			width = COLS-where-(++next2last)->width;
+		else
+			width = it->width*(it->fixed ? 1 : COLS/100.0);
 		
 		switch (it->type)
 		{
@@ -54,6 +60,9 @@ std::string Display::Columns()
 			case 'a':
 				tag = "Artist";
 				break;
+			case 'A':
+				tag = "Album Artist";
+				break;
 			case 't':
 				tag = "Title";
 				break;
@@ -87,7 +96,11 @@ std::string Display::Columns()
 				break;
 		}
 		if (it->right_alignment)
-			for (long i = 0; i < long(width-tag.length()-(it != Config.columns.begin())); ++i, result += ' ') { }
+		{
+			long i = width-tag.length()-(it != Config.columns.begin());
+			if (i > 0)
+				result += std::string(i, ' ');
+		}
 		
 		where += width;
 		result += tag;
@@ -105,20 +118,23 @@ void Display::SongsInColumns(const MPD::Song &s, void *, Menu<MPD::Song> *menu)
 	if (!s.Localized())
 		const_cast<MPD::Song *>(&s)->Localize();
 	
-	bool is_now_playing = menu == myPlaylist->Items && menu->CurrentlyDrawedPosition() == size_t(myPlaylist->NowPlaying);
+	bool is_now_playing = menu == myPlaylist->Items && (menu->isFiltered() ? s.GetPosition() : menu->CurrentlyDrawedPosition()) == size_t(myPlaylist->NowPlaying);
 	if (is_now_playing)
 		*menu << Config.now_playing_prefix;
 	
 	if (Config.columns.empty())
 		return;
 	
-	std::vector<Column>::const_iterator next2last, it;
+	std::vector<Column>::const_iterator next2last, last, it;
 	size_t where = 0;
 	int width;
 	
 	bool last_fixed = Config.columns.back().fixed;
 	if (Config.columns.size() > 1)
 		next2last = Config.columns.end()-2;
+	last = Config.columns.end()-1;
+	
+	bool discard_colors = Config.discard_colors_if_item_is_selected && menu->isSelected(menu->CurrentlyDrawedPosition());
 	
 	for (it = Config.columns.begin(); it != Config.columns.end(); ++it)
 	{
@@ -126,11 +142,17 @@ void Display::SongsInColumns(const MPD::Song &s, void *, Menu<MPD::Song> *menu)
 		{
 			menu->GotoXY(where, menu->Y());
 			*menu << ' ';
-			if ((it-1)->color != clDefault)
+			if (!discard_colors && (it-1)->color != clDefault)
 				*menu << clEnd;
 		}
 		
-		width = last_fixed && it == next2last ? COLS-where-(++next2last)->width : (it->width*(it->fixed ? 1 : COLS/100.0));
+		if (it == Config.columns.end()-1)
+			width = menu->GetWidth()-where;
+		else if (last_fixed && it == next2last)
+			width = COLS-where-(++next2last)->width;
+		else
+			width = it->width*(it->fixed ? 1 : COLS/100.0);
+		
 		MPD::Song::GetFunction get = 0;
 		
 		switch (it->type)
@@ -147,6 +169,9 @@ void Display::SongsInColumns(const MPD::Song &s, void *, Menu<MPD::Song> *menu)
 			case 'a':
 				get = &MPD::Song::GetArtist;
 				break;
+			case 'A':
+				get = &MPD::Song::GetAlbumArtist;
+				break;
 			case 'b':
 				get = &MPD::Song::GetAlbum;
 				break;
@@ -183,13 +208,23 @@ void Display::SongsInColumns(const MPD::Song &s, void *, Menu<MPD::Song> *menu)
 			default:
 				break;
 		}
-		if (it->color != clDefault)
+		if (!discard_colors && it->color != clDefault)
 			*menu << it->color;
 		whline(menu->Raw(), 32, menu->GetWidth()-where);
-		std::string tag = get ? (s.*get)() : "";
+		std::string tag = get ? s.GetTags(get) : "";
+		
+		// last column might need to be shrinked to make space for np/sel suffixes
+		if (it == last)
+		{
+			if (menu->isSelected(menu->CurrentlyDrawedPosition()))
+				width -= Config.selected_item_suffix_length;
+			if (is_now_playing)
+				width -= Config.now_playing_suffix_length;
+		}
+		
 		if (it->right_alignment)
 		{
-			if (!tag.empty() || it->display_empty_tag)
+			if (width > 0 && (!tag.empty() || it->display_empty_tag))
 			{
 				int x, y;
 				menu->GetXY(x, y);
@@ -199,14 +234,29 @@ void Display::SongsInColumns(const MPD::Song &s, void *, Menu<MPD::Song> *menu)
 		}
 		else
 		{
-			if (!tag.empty())
-				*menu << tag;
-			else if (it->display_empty_tag)
-				*menu << Config.empty_tag;
+			if (it == last)
+			{
+				if (width > 0)
+				{
+					std::basic_string<my_char_t> str;
+					if (!tag.empty())
+						str = TO_WSTRING(tag).substr(0, width-1);
+					else if (it->display_empty_tag)
+						str = TO_WSTRING(Config.empty_tag).substr(0, width-1);
+					*menu << str;
+				}
+			}
+			else
+			{
+				if (!tag.empty())
+					*menu << tag;
+				else if (it->display_empty_tag)
+					*menu << Config.empty_tag;
+			}
 		}
 		where += width;
 	}
-	if ((--it)->color != clDefault)
+	if (!discard_colors && (--it)->color != clDefault)
 		*menu << clEnd;
 	if (is_now_playing)
 		*menu << Config.now_playing_suffix;
@@ -217,10 +267,12 @@ void Display::Songs(const MPD::Song &s, void *data, Menu<MPD::Song> *menu)
 	if (!s.Localized())
 		const_cast<MPD::Song *>(&s)->Localize();
 	
-	bool is_now_playing = menu == myPlaylist->Items && menu->CurrentlyDrawedPosition() == size_t(myPlaylist->NowPlaying);
+	bool is_now_playing = menu == myPlaylist->Items && (menu->isFiltered() ? s.GetPosition() : menu->CurrentlyDrawedPosition()) == size_t(myPlaylist->NowPlaying);
 	if (is_now_playing)
 		*menu << Config.now_playing_prefix;
 	
+	bool discard_colors = Config.discard_colors_if_item_is_selected && menu->isSelected(menu->CurrentlyDrawedPosition());
+	
 	std::string line = s.toString(*static_cast<std::string *>(data), "$");
 	for (std::string::const_iterator it = line.begin(); it != line.end(); ++it)
 	{
@@ -233,16 +285,19 @@ void Display::Songs(const MPD::Song &s, void *data, Menu<MPD::Song> *menu)
 			}
 			else if (isdigit(*it)) // color
 			{
-				*menu << Color(*it-'0');
+				if (!discard_colors)
+					*menu << Color(*it-'0');
 			}
 			else if (*it == 'R') // right align
 			{
 				basic_buffer<my_char_t> buf;
 				buf << U(" ");
 				String2Buffer(TO_WSTRING(line.substr(it-line.begin()+1)), buf);
+				if (discard_colors)
+					buf.RemoveFormatting();
 				if (is_now_playing)
 					buf << Config.now_playing_suffix;
-				*menu << XY(menu->GetWidth()-buf.Str().length(), menu->Y()) << buf;
+				*menu << XY(menu->GetWidth()-buf.Str().length()-(menu->isSelected(menu->CurrentlyDrawedPosition()) ? Config.selected_item_suffix_length : 0), menu->Y()) << buf;
 				return;
 			}
 			else // not a color nor right align, just a random character
@@ -264,46 +319,17 @@ void Display::Songs(const MPD::Song &s, void *data, Menu<MPD::Song> *menu)
 
 void Display::Tags(const MPD::Song &s, void *data, Menu<MPD::Song> *menu)
 {
-	switch (static_cast<Menu<std::string> *>(data)->Choice())
+	size_t i = static_cast<Menu<std::string> *>(data)->Choice();
+	if (i < 11)
 	{
-		case 0:
-			ShowTag(*menu, s.GetTitle());
-			return;
-		case 1:
-			ShowTag(*menu, s.GetArtist());
-			return;
-		case 2:
-			ShowTag(*menu, s.GetAlbum());
-			return;
-		case 3:
-			ShowTag(*menu, s.GetDate());
-			return;
-		case 4:
-			ShowTag(*menu, s.GetTrack());
-			return;
-		case 5:
-			ShowTag(*menu, s.GetGenre());
-			return;
-		case 6:
-			ShowTag(*menu, s.GetComposer());
-			return;
-		case 7:
-			ShowTag(*menu, s.GetPerformer());
-			return;
-		case 8:
-			ShowTag(*menu, s.GetDisc());
-			return;
-		case 9:
-			ShowTag(*menu, s.GetComment());
-			return;
-		case 11:
-			if (s.GetNewName().empty())
-				*menu << s.GetName();
-			else
-				*menu << s.GetName() << Config.color2 << " -> " << clEnd << s.GetNewName();
-			return;
-		default:
-			return;
+		ShowTag(*menu, s.GetTags(Info::Tags[i].Get));
+	}
+	else if (i == 12)
+	{
+		if (s.GetNewName().empty())
+			*menu << s.GetName();
+		else
+			*menu << s.GetName() << Config.color2 << " -> " << clEnd << s.GetNewName();
 	}
 }
 
