@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008-2009 by Andrzej Rybczak                            *
+ *   Copyright (C) 2008-2010 by Andrzej Rybczak                            *
  *   electricityispower@gmail.com                                          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -25,14 +25,14 @@
 #include "settings.h"
 #include "tag_editor.h"
 
-using namespace Global;
+using Global::MainHeight;
+using Global::MainStartY;
 
 Help *myHelp = new Help;
 
 void Help::Init()
 {
 	w = new Scrollpad(0, MainStartY, COLS, MainHeight, "", Config.main_color, brNone);
-	w->SetTimeout(ncmpcpp_window_timeout);
 	GetKeybindings();
 	w->Flush();
 	isInitialized = 1;
@@ -47,6 +47,8 @@ void Help::Resize()
 
 void Help::SwitchTo()
 {
+	using Global::myScreen;
+	
 	if (myScreen == this)
 		return;
 	
@@ -56,8 +58,10 @@ void Help::SwitchTo()
 	if (hasToBeResized)
 		Resize();
 	
+	if (myScreen != this && myScreen->isTabbable())
+		Global::myPrevScreen = myScreen;
 	myScreen = this;
-	RedrawHeader = 1;
+	Global::RedrawHeader = 1;
 
 }
 
@@ -134,7 +138,10 @@ void Help::GetKeybindings()
 	*w << DisplayKeys(Key.Home)			<< "Home\n";
 	*w << DisplayKeys(Key.End)			<< "End\n";
 	*w << "\n";
-	*w << DisplayKeys(Key.ScreenSwitcher)		<< "Switch between playlist and browser\n";
+	if (Config.screen_switcher_previous)
+		*w << DisplayKeys(Key.ScreenSwitcher)   << "Switch between current and last screen\n";
+	else
+		*w << DisplayKeys(Key.ScreenSwitcher)   << "Switch between given sequence of screens\n";
 	*w << DisplayKeys(Key.Help)			<< "Help screen\n";
 	*w << DisplayKeys(Key.Playlist)			<< "Playlist screen\n";
 	*w << DisplayKeys(Key.Browser)			<< "Browse screen\n";
@@ -153,7 +160,8 @@ void Help::GetKeybindings()
 #	ifdef ENABLE_CLOCK
 	*w << DisplayKeys(Key.Clock)			<< "Clock screen\n";
 #	endif // ENABLE_CLOCK
-	
+	*w << "\n";
+	*w << DisplayKeys(Key.ServerInfo)		<< "MPD server info\n";
 	
 	*w << "\n\n   " << fmtBold << "Keys - Global\n -----------------------------------------\n" << fmtBoldEnd;
 	*w << DisplayKeys(Key.Stop)			<< "Stop\n";
@@ -176,6 +184,8 @@ void Help::GetKeybindings()
 	*w << DisplayKeys(Key.ToggleRandom)		<< "Toggle random mode\n";
 	*w << DisplayKeys(Key.ToggleSingle)		<< "Toggle single mode\n";
 	*w << DisplayKeys(Key.ToggleConsume)		<< "Toggle consume mode\n";
+	if (Mpd.Version() >= 16)
+		*w << DisplayKeys(Key.ToggleReplayGainMode)	<< "Toggle replay gain mode\n";
 	*w << DisplayKeys(Key.Shuffle)			<< "Shuffle playlist\n";
 	*w << DisplayKeys(Key.ToggleCrossfade)		<< "Toggle crossfade mode\n";
 	*w << DisplayKeys(Key.SetCrossfade)		<< "Set crossfade\n";
@@ -188,6 +198,7 @@ void Help::GetKeybindings()
 	*w << DisplayKeys(Key.NextFoundPosition)	<< "Go to next found position\n";
 	*w << DisplayKeys(Key.ToggleFindMode)		<< "Toggle find mode (normal/wrapped)\n";
 	*w << DisplayKeys(Key.GoToContainingDir)	<< "Locate song in browser\n";
+	*w << DisplayKeys(Key.GoToMediaLibrary)		<< "Locate current song in media library\n";
 	*w << DisplayKeys(Key.ToggleDisplayMode)	<< "Toggle display mode\n";
 	*w << DisplayKeys(Key.ToggleInterface)		<< "Toggle user interface\n";
 	*w << DisplayKeys(Key.GoToPosition)		<< "Go to given position in current song (in % by default)\n";
@@ -203,7 +214,7 @@ void Help::GetKeybindings()
 	
 	*w << "\n\n   " << fmtBold << "Keys - Playlist\n -----------------------------------------\n" << fmtBoldEnd;
 	*w << DisplayKeys(Key.Enter)			<< "Play\n";
-	*w << DisplayKeys(Key.SwitchTagTypeList)	<< "Add random songs to playlist\n";
+	*w << DisplayKeys(Key.SwitchTagTypeList)	<< "Add random songs/artists/albums to playlist\n";
 	*w << DisplayKeys(Key.Delete)			<< "Delete item/selected items from playlist\n";
 	*w << DisplayKeys(Key.Clear)			<< "Clear playlist\n";
 	*w << DisplayKeys(Key.Crop)			<< "Clear playlist but hold currently playing/selected items\n";
@@ -229,7 +240,8 @@ void Help::GetKeybindings()
 	*w << DisplayKeys(Key.EditTags)			<< "Rename playlist/directory\n";
 #	endif // HAVE_TAGLIB_H
 	if (Mpd.GetHostname()[0] == '/') // are we connected to unix socket?
-		*w << DisplayKeys(Key.SwitchTagTypeList)<< "Browse MPD database/local filesystem\n";
+		*w << DisplayKeys(Key.Browser)		<< "Browse MPD database/local filesystem\n";
+	*w << DisplayKeys(Key.SwitchTagTypeList)	<< "Toggle sort order\n";
 	*w << DisplayKeys(Key.GoToNowPlaying)		<< "Locate currently playing song\n";
 	*w << DisplayKeys(Key.GoToParentDir)		<< "Go to parent directory\n";
 	*w << DisplayKeys(Key.Delete)			<< "Delete playlist/file/directory\n";
@@ -242,10 +254,12 @@ void Help::GetKeybindings()
 	*w << DisplayKeys(Key.EditTags)			<< "Edit song's tags\n";
 #	endif // HAVE_TAGLIB_H
 	*w << DisplayKeys(Key.ToggleSingle)		<< "Start searching immediately\n";
+	*w << DisplayKeys(Key.SearchEngine)		<< "Reset search engine\n";
 	
 	
 	*w << "\n\n   " << fmtBold << "Keys - Media library\n -----------------------------------------\n" << fmtBoldEnd;
-	*w << DisplayKeys(Key.MediaLibrary)		<< "Switch between two/three columns\n";
+	if (!Config.media_library_disable_two_column_mode)
+		*w << DisplayKeys(Key.MediaLibrary)	<< "Switch between two/three columns\n";
 	*w << DisplayKeys(&Key.VolumeDown[0], 1)	<< "Previous column\n";
 	*w << DisplayKeys(&Key.VolumeUp[0], 1)		<< "Next column\n";
 	*w << DisplayKeys(Key.Enter)			<< "Add to playlist and play song/album/artist's songs\n";
@@ -274,6 +288,7 @@ void Help::GetKeybindings()
 	*w << "\n\n   " << fmtBold << "Keys - Lyrics\n -----------------------------------------\n" << fmtBoldEnd;
 	*w << DisplayKeys(Key.Space)			<< "Switch for following lyrics of now playing song\n";
 	*w << DisplayKeys(Key.EditTags)			<< "Open lyrics in external editor\n";
+	*w << DisplayKeys(Key.SwitchTagTypeList)	<< "Refetch lyrics\n";
 	
 	
 #	ifdef HAVE_TAGLIB_H
@@ -289,6 +304,7 @@ void Help::GetKeybindings()
 	*w << DisplayKeys(Key.Space)			<< "Select/deselect song (right column)\n";
 	*w << DisplayKeys(&Key.VolumeDown[0], 1)	<< "Previous column\n";
 	*w << DisplayKeys(&Key.VolumeUp[0], 1)		<< "Next column\n";
+	*w << DisplayKeys(Key.GoToParentDir)		<< "Go to parent directory (left column, directories view)\n";
 #	endif // HAVE_TAGLIB_H
 	
 	
