@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008-2009 by Andrzej Rybczak                            *
+ *   Copyright (C) 2008-2010 by Andrzej Rybczak                            *
  *   electricityispower@gmail.com                                          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -30,8 +30,6 @@
 #include "status.h"
 #include "tag_editor.h"
 
-using namespace MPD;
-
 bool ConnectToMPD()
 {
 	if (!Mpd.Connect())
@@ -156,6 +154,8 @@ void ParseArgv(int argc, char **argv)
 					now_playing_format = "{";
 					now_playing_format += argv[i];
 					now_playing_format += "}";
+					Replace(now_playing_format, "\\n", "\n");
+					Replace(now_playing_format, "\\t", "\t");
 					MPD::Song::ValidateFormat("now-playing format", now_playing_format);
 				}
 				std::cout << utf_to_locale_cpy(Mpd.GetCurrentSong().toString(now_playing_format)) << "\n";
@@ -169,24 +169,18 @@ void ParseArgv(int argc, char **argv)
 		}
 		else if (!strcmp(argv[i], "pause"))
 		{
-			Mpd.Execute("pause \"1\"\n");
+			Mpd.Pause(1);
 			quit = 1;
 		}
 		else if (!strcmp(argv[i], "toggle"))
 		{
 			Mpd.UpdateStatus();
-			switch (Mpd.GetState())
+			if (!Mpd.GetErrorMessage().empty())
 			{
-				case psPause:
-				case psPlay:
-					Mpd.Pause();
-					break;
-				case psStop:
-					Mpd.Play();
-					break;
-				default:
-					break;
+				std::cout << "Error: " << Mpd.GetErrorMessage() << std::endl;
+				exit(1);
 			}
+			Mpd.Toggle();
 			quit = 1;
 		}
 		else if (!strcmp(argv[i], "stop"))
@@ -232,18 +226,37 @@ void ParseArgv(int argc, char **argv)
 		exit(0);
 }
 
-bool CaseInsensitiveSorting::operator()(const Item &a, const Item &b)
+int CaseInsensitiveStringComparison::operator()(const std::string &a, const std::string &b)
+{
+	const char *i = a.c_str();
+	const char *j = b.c_str();
+	if (Config.ignore_leading_the)
+	{
+		if (hasTheWord(a))
+			i += 4;
+		if (hasTheWord(b))
+			j += 4;
+	}
+	int dist;
+	while (!(dist = tolower(*i)-tolower(*j)) && *j)
+		++i, ++j;
+	return dist;
+}
+
+bool CaseInsensitiveSorting::operator()(const MPD::Item &a, const MPD::Item &b)
 {
 	if (a.type == b.type)
 	{
 		switch (a.type)
 		{
-			case itDirectory:
+			case MPD::itDirectory:
 				return cmp(ExtractTopDirectory(a.name), ExtractTopDirectory(b.name)) < 0;
-			case itPlaylist:
+			case MPD::itPlaylist:
 				return cmp(a.name, b.name) < 0;
-			case itSong:
-				return operator()(a.song, b.song);
+			case MPD::itSong:
+				return Config.browser_sort_by_mtime
+						? a.song->GetMTime() > b.song->GetMTime()
+						: operator()(a.song, b.song);
 			default: // there's no other type, just silence compiler.
 				return 0;
 		}
@@ -252,7 +265,15 @@ bool CaseInsensitiveSorting::operator()(const Item &a, const Item &b)
 		return a.type < b.type;
 }
 
-void UpdateSongList(Menu<Song> *menu)
+std::string Timestamp(time_t t)
+{
+	char result[32];
+	tm info;
+	result[strftime(result, 31, "%x %X", localtime_r(&t, &info))] = 0;
+	return result;
+}
+
+void UpdateSongList(Menu<MPD::Song> *menu)
 {
 	bool bold = 0;
 	for (size_t i = 0; i < menu->Size(); ++i)
@@ -272,21 +293,21 @@ void UpdateSongList(Menu<Song> *menu)
 }
 
 #ifdef HAVE_TAGLIB_H
-std::string FindSharedDir(Menu<Song> *menu)
+std::string FindSharedDir(Menu<MPD::Song> *menu)
 {
-	SongList list;
+	MPD::SongList list;
 	for (size_t i = 0; i < menu->Size(); ++i)
 		list.push_back(&(*menu)[i]);
 	return FindSharedDir(list);
 }
 
-std::string FindSharedDir(const SongList &v)
+std::string FindSharedDir(const MPD::SongList &v)
 {
 	if (v.empty()) // this should never happen, but in case...
 		FatalError("empty SongList passed to FindSharedDir(const SongList &)!");
 	size_t i = -1;
 	std::string first = v.front()->GetDirectory();
-	for (SongList::const_iterator it = ++v.begin(); it != v.end(); ++it)
+	for (MPD::SongList::const_iterator it = ++v.begin(); it != v.end(); ++it)
 	{
 		size_t j = 0;
 		std::string dir = (*it)->GetDirectory();
@@ -327,8 +348,7 @@ std::string GetLineValue(std::string &line, char a, char b, bool once)
 	}
 	pos[0]++;
 	std::string result = pos[0] >= 0 && pos[1] >= 0 ? line.substr(pos[0], pos[1]-pos[0]) : "";
-	for (i = result.find("\\\""); i != std::string::npos; i = result.find("\\\""))
-		result.replace(i, 2, "\"");
+	Replace(result, "\\\"", "\"");
 	return result;
 }
 
