@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008-2009 by Andrzej Rybczak                            *
+ *   Copyright (C) 2008-2010 by Andrzej Rybczak                            *
  *   electricityispower@gmail.com                                          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -36,6 +36,11 @@ class MediaLibrary : public Screen<Window>
 		std::string Year;
 	};
 	
+	struct SearchConstraintsSorting
+	{
+		bool operator()(const SearchConstraints &a, const SearchConstraints &b) const;
+	};
+	
 	public:
 		virtual void SwitchTo();
 		virtual void Resize();
@@ -48,11 +53,12 @@ class MediaLibrary : public Screen<Window>
 		virtual void EnterPressed() { AddToPlaylist(1); }
 		virtual void SpacePressed();
 		virtual void MouseButtonPressed(MEVENT);
+		virtual bool isTabbable() { return true; }
 		
 		virtual MPD::Song *CurrentSong();
 		
-		virtual bool allowsSelection() { return w == Songs; }
-		virtual void ReverseSelection() { Songs->ReverseSelection(); }
+		virtual bool allowsSelection() { return true; }
+		virtual void ReverseSelection();
 		virtual void GetSelectedSongs(MPD::SongList &);
 		
 		virtual void ApplyFilter(const std::string &);
@@ -63,8 +69,10 @@ class MediaLibrary : public Screen<Window>
 		void NextColumn();
 		void PrevColumn();
 		
+		void LocateSong(const MPD::Song &);
+		
 		Menu<std::string> *Artists;
-		Menu< std::pair<std::string, SearchConstraints> > *Albums;
+		Menu<SearchConstraints> *Albums;
 		Menu<MPD::Song> *Songs;
 		
 	protected:
@@ -75,8 +83,10 @@ class MediaLibrary : public Screen<Window>
 		
 		static std::string SongToString(const MPD::Song &s, void *);
 		
+		static std::string AlbumToString(const SearchConstraints &, void *);
+		static void DisplayAlbums(const SearchConstraints &, void *, Menu<SearchConstraints> *);
+		
 		static bool SortSongsByTrack(MPD::Song *, MPD::Song *);
-		static bool SortSongsByYear(MPD::Song *, MPD::Song *);
 		
 		static bool hasTwoColumns;
 		static size_t itsLeftColWidth;
