@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008-2009 by Andrzej Rybczak                            *
+ *   Copyright (C) 2008-2010 by Andrzej Rybczak                            *
  *   electricityispower@gmail.com                                          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -28,7 +28,8 @@
 #include "song.h"
 #include "status.h"
 
-using namespace Global;
+using Global::MainHeight;
+using Global::MainStartY;
 
 Playlist *myPlaylist = new Playlist;
 
@@ -47,8 +48,8 @@ Menu< std::pair<std::string, MPD::Song::GetFunction> > *Playlist::SortDialog = 0
 void Playlist::Init()
 {
 	Items = new Menu<MPD::Song>(0, MainStartY, COLS, MainHeight, Config.columns_in_playlist ? Display::Columns() : "", Config.main_color, brNone);
-	Items->SetTimeout(ncmpcpp_window_timeout);
 	Items->CyclicScrolling(Config.use_cyclic_scrolling);
+	Items->CenteredCursor(Config.centered_cursor);
 	Items->HighlightColor(Config.main_highlight_color);
 	Items->SetSelectPrefix(&Config.selected_item_prefix);
 	Items->SetSelectSuffix(&Config.selected_item_suffix);
@@ -62,8 +63,8 @@ void Playlist::Init()
 		SortDialogHeight = std::min(int(MainHeight), 18);
 		
 		SortDialog = new Menu< std::pair<std::string, MPD::Song::GetFunction> >((COLS-SortDialogWidth)/2, (MainHeight-SortDialogHeight)/2+MainStartY, SortDialogWidth, SortDialogHeight, "Sort songs by...", Config.main_color, Config.window_border);
-		SortDialog->SetTimeout(ncmpcpp_window_timeout);
 		SortDialog->CyclicScrolling(Config.use_cyclic_scrolling);
+		SortDialog->CenteredCursor(Config.centered_cursor);
 		SortDialog->SetItemDisplayer(Display::Pairs);
 		
 		SortDialog->AddOption(std::make_pair("Artist", &MPD::Song::GetArtist));
@@ -88,6 +89,8 @@ void Playlist::Init()
 
 void Playlist::SwitchTo()
 {
+	using Global::myScreen;
+	
 	if (myScreen == this)
 		return;
 	
@@ -99,12 +102,14 @@ void Playlist::SwitchTo()
 	if (hasToBeResized)
 		Resize();
 	
+	if (myScreen != this && myScreen->isTabbable())
+		Global::myPrevScreen = myScreen;
 	myScreen = this;
 	Items->Window::Clear();
 	EnableHighlighting();
 	if (w != Items) // even if sorting window is active, background has to be refreshed anyway
 		Items->Display();
-	RedrawHeader = 1;
+	Global::RedrawHeader = 1;
 }
 
 void Playlist::Resize()
@@ -132,7 +137,7 @@ std::basic_string<my_char_t> Playlist::Title()
 	std::basic_string<my_char_t> result = U("Playlist ");
 	if (ReloadTotalLength || ReloadRemaining)
 		itsBufferedStats = TotalLength();
-	result += Scroller(TO_WSTRING(itsBufferedStats), itsScrollBegin, Items->GetWidth()-result.length()-(Config.new_design ? 2 : VolumeState.length()));
+	result += Scroller(TO_WSTRING(itsBufferedStats), itsScrollBegin, Items->GetWidth()-result.length()-(Config.new_design ? 2 : Global::VolumeState.length()));
 	return result;
 }
 
@@ -143,12 +148,23 @@ void Playlist::EnterPressed()
 		if (!Items->Empty())
 		{
 			Mpd.PlayID(Items->Current().GetID());
-			UpdateStatusImmediately = 1;
+			Global::UpdateStatusImmediately = 1;
 		}
 	}
 	else if (w == SortDialog)
 	{
 		size_t pos = SortDialog->Choice();
+		
+		size_t beginning = 0;
+		size_t end = Items->Size();
+		if (Items->hasSelected())
+		{
+			std::vector<size_t> list;
+			Items->GetSelected(list);
+			beginning = *list.begin();
+			end = *list.rbegin()+1;
+		}
+		
 		if (pos > SortOptions)
 		{
 			if (pos == SortOptions+2) // reverse
@@ -156,7 +172,7 @@ void Playlist::EnterPressed()
 				BlockUpdate = 1;
 				ShowMessage("Reversing playlist order...");
 				Mpd.StartCommandsList();
-				for (size_t i = 0, j = Items->Size()-1; i < Items->Size()/2; ++i, --j)
+				for (size_t i = beginning, j = end-1; i < (beginning+end)/2; ++i, --j)
 				{
 					Mpd.Swap(i, j);
 					Items->Swap(i, j);
@@ -180,8 +196,8 @@ void Playlist::EnterPressed()
 		ShowMessage("Sorting playlist...");
 		MPD::SongList playlist, cmp;
 		
-		playlist.reserve(Items->Size());
-		for (size_t i = 0; i < Items->Size(); ++i)
+		playlist.reserve(end-beginning);
+		for (size_t i = beginning; i < end; ++i)
 		{
 			(*Items)[i].SetPosition(i);
 			playlist.push_back(&(*Items)[i]);
@@ -199,15 +215,15 @@ void Playlist::EnterPressed()
 		Mpd.StartCommandsList();
 		do
 		{
-			for (size_t i = 0; i < playlist.size(); ++i)
+			for (size_t i = 0, j = beginning; i < playlist.size(); ++i, ++j)
 			{
-				if (playlist[i]->GetPosition() > int(i))
+				if (playlist[i]->GetPosition() > j)
 				{
-					Mpd.Swap(playlist[i]->GetPosition(), i);
-					std::swap(cmp[playlist[i]->GetPosition()], cmp[i]);
-					Items->Swap(playlist[i]->GetPosition(), i);
+					Mpd.Swap(playlist[i]->GetPosition(), j);
+					std::swap(cmp[playlist[i]->GetPosition()-beginning], cmp[i]);
+					Items->Swap(playlist[i]->GetPosition(), j);
 				}
-				cmp[i]->SetPosition(i);
+				cmp[i]->SetPosition(j);
 			}
 		}
 		while (playlist != cmp);
@@ -218,13 +234,19 @@ void Playlist::EnterPressed()
 
 void Playlist::SpacePressed()
 {
-	if (w == Items)
+	if (w == Items && !Items->Empty())
 	{
 		Items->Select(Items->Choice(), !Items->isSelected());
 		Items->Scroll(wDown);
 	}
 }
 
+void Playlist::ReadKey(int &key)
+{
+	w->ReadKey(key);
+	UpdateTimer();
+}
+
 void Playlist::MouseButtonPressed(MEVENT me)
 {
 	if (w == Items && !Items->Empty() && Items->hasCoords(me.x, me.y))
@@ -260,6 +282,8 @@ void Playlist::GetSelectedSongs(MPD::SongList &v)
 {
 	std::vector<size_t> selected;
 	Items->GetSelected(selected);
+	if (selected.empty())
+		selected.push_back(Items->Choice());
 	for (std::vector<size_t>::const_iterator it = selected.begin(); it != selected.end(); ++it)
 		v.push_back(new MPD::Song(Items->at(*it)));
 }
@@ -323,7 +347,7 @@ bool Playlist::Sorting(MPD::Song *a, MPD::Song *b)
 {
 	CaseInsensitiveStringComparison cmp;
 	for (size_t i = 0; i < SortOptions; ++i)
-		if (int ret = cmp((a->*(*SortDialog)[i].second)(), (b->*(*SortDialog)[i].second)()))
+		if (int ret = cmp(a->GetTags((*SortDialog)[i].second), b->GetTags((*SortDialog)[i].second)))
 			return ret < 0;
 	return a->GetPosition() < b->GetPosition();
 }
@@ -361,60 +385,17 @@ std::string Playlist::TotalLength()
 	if (itsTotalLength)
 	{
 		result << ", length: ";
-		ShowTime(result, itsTotalLength);
+		ShowTime(result, itsTotalLength, 0);
 	}
 	if (Config.playlist_show_remaining_time && itsRemainingTime && !Items->isFiltered() && Items->Size() > 1)
 	{
 		result << " :: remaining: ";
-		ShowTime(result, itsRemainingTime);
+		ShowTime(result, itsRemainingTime, 0);
 	}
 	result << ')';
 	return result.str();
 }
 
-void Playlist::ShowTime(std::ostringstream &result, size_t length)
-{
-	const int MINUTE = 60;
-	const int HOUR = 60*MINUTE;
-	const int DAY = 24*HOUR;
-	const int YEAR = 365*DAY;
-	
-	int years = length/YEAR;
-	if (years)
-	{
-		result << years << (years == 1 ? " year" : " years");
-		length -= years*YEAR;
-		if (length)
-			result << ", ";
-	}
-	int days = length/DAY;
-	if (days)
-	{
-		result << days << (days == 1 ? " day" : " days");
-		length -= days*DAY;
-		if (length)
-			result << ", ";
-	}
-	int hours = length/HOUR;
-	if (hours)
-	{
-		result << hours << (hours == 1 ? " hour" : " hours");
-		length -= hours*HOUR;
-		if (length)
-			result << ", ";
-	}
-	int minutes = length/MINUTE;
-	if (minutes)
-	{
-		result << minutes << (minutes == 1 ? " minute" : " minutes");
-		length -= minutes*MINUTE;
-		if (length)
-			result << ", ";
-	}
-	if (length)
-		result << length << (length == 1 ? " second" : " seconds");
-}
-
 const MPD::Song *Playlist::NowPlayingSong()
 {
 	bool was_filtered = Items->isFiltered();
@@ -453,9 +434,9 @@ std::string Playlist::SongInColumnsToString(const MPD::Song &s, void *)
 	return s.toString(result);
 }
 
-bool Playlist::Add(const MPD::Song &s, bool in_playlist, bool play)
+bool Playlist::Add(const MPD::Song &s, bool in_playlist, bool play, int position)
 {
-	BlockItemListUpdate = 1;
+	Global::BlockItemListUpdate = 1;
 	if (Config.ncmpc_like_songs_adding && in_playlist)
 	{
 		unsigned hash = s.GetHash();
@@ -491,7 +472,7 @@ bool Playlist::Add(const MPD::Song &s, bool in_playlist, bool play)
 	}
 	else
 	{
-		int id = Mpd.AddSong(s);
+		int id = Mpd.AddSong(s, position);
 		if (id >= 0)
 		{
 			ShowMessage("Added to playlist: %s", s.toString(Config.song_status_format_no_colors).c_str());
@@ -504,7 +485,7 @@ bool Playlist::Add(const MPD::Song &s, bool in_playlist, bool play)
 	}
 }
 
-bool Playlist::Add(const MPD::SongList &l, bool play)
+bool Playlist::Add(const MPD::SongList &l, bool play, int position)
 {
 	if (l.empty())
 		return false;
@@ -513,15 +494,25 @@ bool Playlist::Add(const MPD::SongList &l, bool play)
 	
 	Mpd.StartCommandsList();
 	MPD::SongList::const_iterator it = l.begin();
-	for (; it != l.end(); ++it)
-		if (Mpd.AddSong(**it) < 0)
-			break;
+	if (position < 0)
+	{
+		for (; it != l.end(); ++it)
+			if (Mpd.AddSong(**it) < 0)
+				break;
+	}
+	else
+	{
+		MPD::SongList::const_reverse_iterator j = l.rbegin();
+		for (; j != l.rend(); ++j)
+			if (Mpd.AddSong(**j, position) < 0)
+				break;
+	}
 	Mpd.CommitCommandsList();
 	
 	if (play && old_playlist_size < Items->Size())
 		Mpd.Play(old_playlist_size);
 	
-	if (Items->Back().GetHash() != l.back()->GetHash())
+	if (position < 0 && Items->Back().GetHash() != l.back()->GetHash())
 	{
 		if (it != l.begin())
 			ShowMessage("%s", MPD::Message::PartOfSongsAdded);
