@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008-2009 by Andrzej Rybczak                            *
+ *   Copyright (C) 2008-2010 by Andrzej Rybczak                            *
  *   electricityispower@gmail.com                                          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -30,8 +30,8 @@
 #include "status.h"
 #include "tag_editor.h"
 
-using namespace Global;
-using namespace MPD;
+using Global::MainHeight;
+using Global::MainStartY;
 
 PlaylistEditor *myPlaylistEditor = new PlaylistEditor;
 
@@ -47,14 +47,14 @@ void PlaylistEditor::Init()
 	
 	Playlists = new Menu<std::string>(0, MainStartY, LeftColumnWidth, MainHeight, "Playlists", Config.main_color, brNone);
 	Playlists->HighlightColor(Config.active_column_color);
-	Playlists->SetTimeout(ncmpcpp_window_timeout);
 	Playlists->CyclicScrolling(Config.use_cyclic_scrolling);
+	Playlists->CenteredCursor(Config.centered_cursor);
 	Playlists->SetItemDisplayer(Display::Generic);
 	
-	Content = new Menu<Song>(RightColumnStartX, MainStartY, RightColumnWidth, MainHeight, "Playlist's content", Config.main_color, brNone);
+	Content = new Menu<MPD::Song>(RightColumnStartX, MainStartY, RightColumnWidth, MainHeight, "Playlist's content", Config.main_color, brNone);
 	Content->HighlightColor(Config.main_highlight_color);
-	Content->SetTimeout(ncmpcpp_window_timeout);
 	Content->CyclicScrolling(Config.use_cyclic_scrolling);
+	Content->CenteredCursor(Config.centered_cursor);
 	Content->SetSelectPrefix(&Config.selected_item_prefix);
 	Content->SetSelectSuffix(&Config.selected_item_suffix);
 	Content->SetItemDisplayer(Display::Songs);
@@ -95,6 +95,8 @@ void PlaylistEditor::Refresh()
 
 void PlaylistEditor::SwitchTo()
 {
+	using Global::myScreen;
+	
 	if (myScreen == this)
 		return;
 	
@@ -104,8 +106,10 @@ void PlaylistEditor::SwitchTo()
 	if (hasToBeResized)
 		Resize();
 	
+	if (myScreen != this && myScreen->isTabbable())
+		Global::myPrevScreen = myScreen;
 	myScreen = this;
-	RedrawHeader = 1;
+	Global::RedrawHeader = 1;
 	Refresh();
 	UpdateSongList(Content);
 }
@@ -114,11 +118,11 @@ void PlaylistEditor::Update()
 {
 	if (Playlists->Empty())
 	{
-		Content->Clear(0);
-		TagList list;
+		Content->Clear();
+		MPD::TagList list;
 		Mpd.GetPlaylists(list);
 		sort(list.begin(), list.end(), CaseInsensitiveSorting());
-		for (TagList::iterator it = list.begin(); it != list.end(); ++it)
+		for (MPD::TagList::iterator it = list.begin(); it != list.end(); ++it)
 		{
 			utf_to_locale(*it);
 			Playlists->AddOption(*it);
@@ -130,14 +134,14 @@ void PlaylistEditor::Update()
 	if (!Playlists->Empty() && Content->Empty())
 	{
 		Content->Reset();
-		SongList list;
+		MPD::SongList list;
 		Mpd.GetPlaylistContent(locale_to_utf_cpy(Playlists->Current()), list);
 		if (!list.empty())
 			Content->SetTitle("Playlist's content (" + IntoStr(list.size()) + " item" + (list.size() == 1 ? ")" : "s)"));
 		else
 			Content->SetTitle("Playlist's content");
 		bool bold = 0;
-		for (SongList::const_iterator it = list.begin(); it != list.end(); ++it)
+		for (MPD::SongList::const_iterator it = list.begin(); it != list.end(); ++it)
 		{
 			for (size_t j = 0; j < myPlaylist->Items->Size(); ++j)
 			{
@@ -150,7 +154,7 @@ void PlaylistEditor::Update()
 			Content->AddOption(**it, bold);
 			bold = 0;
 		}
-		FreeSongList(list);
+		MPD::FreeSongList(list);
 		Content->Window::Clear();
 		Content->Display();
 	}
@@ -193,7 +197,7 @@ void PlaylistEditor::PrevColumn()
 
 void PlaylistEditor::AddToPlaylist(bool add_n_play)
 {
-	SongList list;
+	MPD::SongList list;
 	
 	if (w == Playlists && !Playlists->Empty())
 	{
@@ -239,7 +243,7 @@ void PlaylistEditor::MouseButtonPressed(MEVENT me)
 		}
 		else
 			Screen<Window>::MouseButtonPressed(me);
-		Content->Clear(0);
+		Content->Clear();
 	}
 	else if (!Content->Empty() && Content->hasCoords(me.x, me.y))
 	{
@@ -272,6 +276,8 @@ void PlaylistEditor::GetSelectedSongs(MPD::SongList &v)
 {
 	std::vector<size_t> selected;
 	Content->GetSelected(selected);
+	if (selected.empty())
+		selected.push_back(Content->Choice());
 	for (std::vector<size_t>::const_iterator it = selected.begin(); it != selected.end(); ++it)
 	{
 		v.push_back(new MPD::Song(Content->at(*it)));
