@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008-2009 by Andrzej Rybczak                            *
+ *   Copyright (C) 2008-2010 by Andrzej Rybczak                            *
  *   electricityispower@gmail.com                                          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -118,6 +118,11 @@ class BasicScreen
 		/// cast to List if available or null pointer otherwise
 		///
 		virtual List *GetList() = 0;
+
+		/// When this is overwritten with a function returning true, the
+		/// screen will be used in tab switching.
+		///
+		virtual bool isTabbable() { return false; }
 		
 		/// Should be set to true each time screen needs resize
 		///
@@ -229,7 +234,7 @@ template <typename WindowType> void Screen<WindowType>::Scroll(Where where, cons
 			TraceMpdStatus();
 			w->Scroll(where);
 			w->Refresh();
-			w->ReadKey(in);
+			ReadKey(in);
 		}
 		w->SetTimeout(ncmpcpp_window_timeout);
 	}
