@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008-2009 by Andrzej Rybczak                            *
+ *   Copyright (C) 2008-2010 by Andrzej Rybczak                            *
  *   electricityispower@gmail.com                                          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -28,8 +28,8 @@
 #include "settings.h"
 #include "status.h"
 
-using namespace MPD;
-using namespace Global;
+using Global::MainHeight;
+using Global::MainStartY;
 
 SearchEngine *mySearcher = new SearchEngine;
 
@@ -47,26 +47,29 @@ const char *SearchEngine::ConstraintsNames[] =
 	"Comment:"
 };
 
-const char *SearchEngine::NormalMode = "Match if tag contains searched phrase (regexes supported)";
-const char *SearchEngine::StrictMode = "Match only if both values are the same";
+const char *SearchEngine::SearchModes[] =
+{
+	"Match if tag contains searched phrase (no regexes)",
+	"Match if tag contains searched phrase (regexes supported)",
+	"Match only if both values are the same",
+	0
+};
 
-size_t SearchEngine::StaticOptions = 20;
-size_t SearchEngine::SearchButton = 15;
-size_t SearchEngine::ResetButton = 16;
-
-bool SearchEngine::MatchToPattern = 1;
-int SearchEngine::CaseSensitive = REG_ICASE;
+size_t SearchEngine::StaticOptions = 19;
+size_t SearchEngine::ResetButton = 15;
+size_t SearchEngine::SearchButton = 14;
 
 void SearchEngine::Init()
 {
-	w = new Menu< std::pair<Buffer *, Song *> >(0, MainStartY, COLS, MainHeight, "", Config.main_color, brNone);
+	w = new Menu< std::pair<Buffer *, MPD::Song *> >(0, MainStartY, COLS, MainHeight, "", Config.main_color, brNone);
 	w->HighlightColor(Config.main_highlight_color);
-	w->SetTimeout(ncmpcpp_window_timeout);
 	w->CyclicScrolling(Config.use_cyclic_scrolling);
+	w->CenteredCursor(Config.centered_cursor);
 	w->SetItemDisplayer(Display::SearchEngine);
 	w->SetSelectPrefix(&Config.selected_item_prefix);
 	w->SetSelectSuffix(&Config.selected_item_suffix);
 	w->SetGetStringFunction(SearchEngineOptionToString);
+	SearchMode = &SearchModes[Config.search_engine_default_search_mode];
 	isInitialized = 1;
 }
 
@@ -79,8 +82,13 @@ void SearchEngine::Resize()
 
 void SearchEngine::SwitchTo()
 {
+	using Global::myScreen;
+	
 	if (myScreen == this)
+	{
+		Reset();
 		return;
+	}
 	
 	if (!isInitialized)
 		Init();
@@ -90,8 +98,11 @@ void SearchEngine::SwitchTo()
 	
 	if (w->Empty())
 		Prepare();
+
+	if (myScreen != this && myScreen->isTabbable())
+		Global::myPrevScreen = myScreen;
 	myScreen = this;
-	RedrawHeader = 1;
+	Global::RedrawHeader = 1;
 	
 	if (!w->Back().first)
 	{
@@ -108,34 +119,31 @@ std::basic_string<my_char_t> SearchEngine::Title()
 void SearchEngine::EnterPressed()
 {
 	size_t option = w->Choice();
-	if (option < SearchButton)
+	if (option > ConstraintsNumber && option < SearchButton)
 		w->Current().first->Clear();
-	if (option < 15)
+	if (option < SearchButton)
 		LockStatusbar();
 	
-	if (option < 10)
+	if (option < ConstraintsNumber)
 	{
 		Statusbar() << fmtBold << ConstraintsNames[option] << fmtBoldEnd << ' ';
-		itsConstraints[option] = wFooter->GetString(itsConstraints[option]);
+		itsConstraints[option] = Global::wFooter->GetString(itsConstraints[option]);
+		w->Current().first->Clear();
 		*w->Current().first << fmtBold << std::setw(10) << std::left << ConstraintsNames[option] << fmtBoldEnd << ' ';
 		ShowTag(*w->Current().first, itsConstraints[option]);
 	}
-	else if (option == 11)
+	else if (option == ConstraintsNumber+1)
 	{
 		Config.search_in_db = !Config.search_in_db;
 		*w->Current().first << fmtBold << "Search in:" << fmtBoldEnd << ' ' << (Config.search_in_db ? "Database" : "Current playlist");
 	}
-	else if (option == 12)
+	else if (option == ConstraintsNumber+2)
 	{
-		MatchToPattern = !MatchToPattern;
-		*w->Current().first << fmtBold << "Search mode:" << fmtBoldEnd << ' ' << (MatchToPattern ? NormalMode : StrictMode);
+		if (!*++SearchMode)
+			SearchMode = &SearchModes[0];
+		*w->Current().first << fmtBold << "Search mode:" << fmtBoldEnd << ' ' << *SearchMode;
 	}
-	else if (option == 13)
-	{
-		CaseSensitive = !CaseSensitive * REG_ICASE;
-		*w->Current().first << fmtBold << "Case sensitive:" << fmtBoldEnd << ' ' << (!CaseSensitive ? "Yes" : "No");
-	}
-	else if (option == 15)
+	else if (option == SearchButton)
 	{
 		ShowMessage("Searching...");
 		if (w->Size() > StaticOptions)
@@ -148,7 +156,7 @@ void SearchEngine::EnterPressed()
 			size_t found = w->Size()-SearchEngine::StaticOptions;
 			found += 3; // don't count options inserted below
 			w->InsertSeparator(ResetButton+1);
-			w->InsertOption(ResetButton+2, std::make_pair(static_cast<Buffer *>(0), static_cast<Song *>(0)), 1, 1);
+			w->InsertOption(ResetButton+2, std::make_pair(static_cast<Buffer *>(0), static_cast<MPD::Song *>(0)), 1, 1);
 			w->at(ResetButton+2).first = new Buffer();
 			*w->at(ResetButton+2).first << Config.color1 << "Search results: " << Config.color2 << "Found " << found  << (found > 1 ? " songs" : " song") << clDefault;
 			w->InsertSeparator(ResetButton+3);
@@ -163,18 +171,14 @@ void SearchEngine::EnterPressed()
 		else
 			ShowMessage("No results found");
 	}
-	else if (option == 16)
+	else if (option == ResetButton)
 	{
-		for (size_t i = 0; i < ConstraintsNumber; ++i)
-			itsConstraints[i].clear();
-		w->Reset();
-		Prepare();
-		ShowMessage("Search state reset");
+		Reset();
 	}
 	else
 		w->Bold(w->Choice(), myPlaylist->Add(*w->Current().second, w->isBold(), 1));
 	
-	if (option < 15)
+	if (option < SearchButton)
 		UnlockStatusbar();
 }
 
@@ -203,7 +207,7 @@ void SearchEngine::MouseButtonPressed(MEVENT me)
 		if (!w->Goto(me.y))
 			return;
 		w->Refresh();
-		if ((me.bstate & BUTTON3_PRESSED || w->Choice() > 10) && w->Choice() < StaticOptions)
+		if ((me.bstate & BUTTON3_PRESSED || w->Choice() > ConstraintsNumber) && w->Choice() < StaticOptions)
 			EnterPressed();
 		else if (w->Choice() >= StaticOptions)
 		{
@@ -231,6 +235,8 @@ void SearchEngine::GetSelectedSongs(MPD::SongList &v)
 {
 	std::vector<size_t> selected;
 	w->GetSelected(selected);
+	if (selected.empty() && w->Choice() >= StaticOptions)
+		selected.push_back(w->Choice());
 	for (std::vector<size_t>::const_iterator it = selected.begin(); it != selected.end(); ++it)
 		v.push_back(new MPD::Song(*w->at(*it).second));
 }
@@ -262,22 +268,22 @@ void SearchEngine::Prepare()
 {
 	for (size_t i = 0; i < w->Size(); ++i)
 	{
-		if (i == 10 || i == 14 || i == ResetButton+1 || i == ResetButton+3) // separators
+		if (i == ConstraintsNumber || i == SearchButton-1 || i == ResetButton+1 || i == ResetButton+3) // separators
 			continue;
 		delete (*w)[i].first;
 		delete (*w)[i].second;
 	}
 	
 	w->SetTitle("");
-	w->Clear(0);
-	w->ResizeList(17);
+	w->Clear();
+	w->ResizeList(StaticOptions-3);
 	
-	w->IntoSeparator(10);
-	w->IntoSeparator(14);
+	w->IntoSeparator(ConstraintsNumber);
+	w->IntoSeparator(SearchButton-1);
 	
-	for (size_t i = 0; i < 17; ++i)
+	for (size_t i = 0; i < StaticOptions-3; ++i)
 	{
-		if (i == 10 || i == 14) // separators
+		if (i == ConstraintsNumber || i == SearchButton-1) // separators
 			continue;
 		(*w)[i].first = new Buffer();
 	}
@@ -288,12 +294,20 @@ void SearchEngine::Prepare()
 		ShowTag(*(*w)[i].first, itsConstraints[i]);
 	}
 	
-	*w->at(11).first << fmtBold << "Search in:" << fmtBoldEnd << ' ' << (Config.search_in_db ? "Database" : "Current playlist");
-	*w->at(12).first << fmtBold << "Search mode:" << fmtBoldEnd << ' ' << (MatchToPattern ? NormalMode : StrictMode);
-	*w->at(13).first << fmtBold << "Case sensitive:" << fmtBoldEnd << ' ' << (!CaseSensitive ? "Yes" : "No");
+	*w->at(ConstraintsNumber+1).first << fmtBold << "Search in:" << fmtBoldEnd << ' ' << (Config.search_in_db ? "Database" : "Current playlist");
+	*w->at(ConstraintsNumber+2).first << fmtBold << "Search mode:" << fmtBoldEnd << ' ' << *SearchMode;
 	
-	*w->at(15).first << "Search";
-	*w->at(16).first << "Reset";
+	*w->at(SearchButton).first << "Search";
+	*w->at(ResetButton).first << "Reset";
+}
+
+void SearchEngine::Reset()
+{
+	for (size_t i = 0; i < ConstraintsNumber; ++i)
+		itsConstraints[i].clear();
+	w->Reset();
+	Prepare();
+	ShowMessage("Search state reset");
 }
 
 void SearchEngine::Search()
@@ -310,7 +324,37 @@ void SearchEngine::Search()
 	if (constraints_empty)
 		return;
 	
-	SongList list;
+	if (Config.search_in_db && (SearchMode == &SearchModes[0] || SearchMode == &SearchModes[2])) // use built-in mpd searching
+	{
+		Mpd.StartSearch(SearchMode == &SearchModes[2]);
+		if (!itsConstraints[0].empty())
+			Mpd.AddSearchAny(itsConstraints[0]);
+		if (!itsConstraints[1].empty())
+			Mpd.AddSearch(MPD_TAG_ARTIST, itsConstraints[1]);
+		if (!itsConstraints[2].empty())
+			Mpd.AddSearch(MPD_TAG_TITLE, itsConstraints[2]);
+		if (!itsConstraints[3].empty())
+			Mpd.AddSearch(MPD_TAG_ALBUM, itsConstraints[3]);
+		if (!itsConstraints[4].empty())
+			Mpd.AddSearchURI(itsConstraints[4]);
+		if (!itsConstraints[5].empty())
+			Mpd.AddSearch(MPD_TAG_COMPOSER, itsConstraints[5]);
+		if (!itsConstraints[6].empty())
+			Mpd.AddSearch(MPD_TAG_PERFORMER, itsConstraints[6]);
+		if (!itsConstraints[7].empty())
+			Mpd.AddSearch(MPD_TAG_GENRE, itsConstraints[7]);
+		if (!itsConstraints[8].empty())
+			Mpd.AddSearch(MPD_TAG_DATE, itsConstraints[8]);
+		if (!itsConstraints[9].empty())
+			Mpd.AddSearch(MPD_TAG_COMMENT, itsConstraints[9]);
+		MPD::SongList results;
+		Mpd.CommitSearch(results);
+		for (MPD::SongList::const_iterator it = results.begin(); it != results.end(); ++it)
+			w->AddOption(std::make_pair(static_cast<Buffer *>(0), *it));
+		return;
+	}
+	
+	MPD::SongList list;
 	if (Config.search_in_db)
 		Mpd.GetDirectoryRecursive("/", list);
 	else
@@ -323,14 +367,14 @@ void SearchEngine::Search()
 	bool any_found = 1;
 	bool found = 1;
 	
-	for (SongList::const_iterator it = list.begin(); it != list.end(); ++it)
+	for (MPD::SongList::const_iterator it = list.begin(); it != list.end(); ++it)
 	{
-		if (MatchToPattern)
+		if (SearchMode != &SearchModes[2]) // match to pattern
 		{
 			regex_t rx;
 			if (!itsConstraints[0].empty())
 			{
-				if (regcomp(&rx, itsConstraints[0].c_str(), CaseSensitive | Config.regex_type) == 0)
+				if (regcomp(&rx, itsConstraints[0].c_str(), REG_ICASE | Config.regex_type) == 0)
 				{
 					any_found =
 						!regexec(&rx, (*it)->GetArtist().c_str(), 0, 0, 0)
@@ -348,96 +392,98 @@ void SearchEngine::Search()
 			
 			if (found && !itsConstraints[1].empty())
 			{
-				if (!regcomp(&rx, itsConstraints[1].c_str(), CaseSensitive | Config.regex_type))
+				if (!regcomp(&rx, itsConstraints[1].c_str(), REG_ICASE | Config.regex_type))
 					found = !regexec(&rx, (*it)->GetArtist().c_str(), 0, 0, 0);
 				regfree(&rx);
 			}
 			if (found && !itsConstraints[2].empty())
 			{
-				if (!regcomp(&rx, itsConstraints[2].c_str(), CaseSensitive | Config.regex_type))
+				if (!regcomp(&rx, itsConstraints[2].c_str(), REG_ICASE | Config.regex_type))
 					found = !regexec(&rx, (*it)->GetTitle().c_str(), 0, 0, 0);
 				regfree(&rx);
 			}
 			if (found && !itsConstraints[3].empty())
 			{
-				if (!regcomp(&rx, itsConstraints[3].c_str(), CaseSensitive | Config.regex_type))
+				if (!regcomp(&rx, itsConstraints[3].c_str(), REG_ICASE | Config.regex_type))
 					found = !regexec(&rx, (*it)->GetAlbum().c_str(), 0, 0, 0);
 				regfree(&rx);
 			}
 			if (found && !itsConstraints[4].empty())
 			{
-				if (!regcomp(&rx, itsConstraints[4].c_str(), CaseSensitive | Config.regex_type))
+				if (!regcomp(&rx, itsConstraints[4].c_str(), REG_ICASE | Config.regex_type))
 					found = !regexec(&rx, (*it)->GetName().c_str(), 0, 0, 0);
 				regfree(&rx);
 			}
 			if (found && !itsConstraints[5].empty())
 			{
-				if (!regcomp(&rx, itsConstraints[5].c_str(), CaseSensitive | Config.regex_type))
+				if (!regcomp(&rx, itsConstraints[5].c_str(), REG_ICASE | Config.regex_type))
 					found = !regexec(&rx, (*it)->GetComposer().c_str(), 0, 0, 0);
 				regfree(&rx);
 			}
 			if (found && !itsConstraints[6].empty())
 			{
-				if (!regcomp(&rx, itsConstraints[6].c_str(), CaseSensitive | Config.regex_type))
+				if (!regcomp(&rx, itsConstraints[6].c_str(), REG_ICASE | Config.regex_type))
 					found = !regexec(&rx, (*it)->GetPerformer().c_str(), 0, 0, 0);
 				regfree(&rx);
 			}
 			if (found && !itsConstraints[7].empty())
 			{
-				if (!regcomp(&rx, itsConstraints[7].c_str(), CaseSensitive | Config.regex_type))
+				if (!regcomp(&rx, itsConstraints[7].c_str(), REG_ICASE | Config.regex_type))
 					found = !regexec(&rx, (*it)->GetGenre().c_str(), 0, 0, 0);
 				regfree(&rx);
 			}
 			if (found && !itsConstraints[8].empty())
 			{
-				if (!regcomp(&rx, itsConstraints[8].c_str(), CaseSensitive | Config.regex_type))
+				if (!regcomp(&rx, itsConstraints[8].c_str(), REG_ICASE | Config.regex_type))
 					found = !regexec(&rx, (*it)->GetDate().c_str(), 0, 0, 0);
 				regfree(&rx);
 			}
 			if (found && !itsConstraints[9].empty())
 			{
-				if (!regcomp(&rx, itsConstraints[9].c_str(), CaseSensitive | Config.regex_type))
+				if (!regcomp(&rx, itsConstraints[9].c_str(), REG_ICASE | Config.regex_type))
 					found = !regexec(&rx, (*it)->GetComment().c_str(), 0, 0, 0);
 				regfree(&rx);
 			}
 		}
-		else
+		else // match only if values are equal
 		{
+			CaseInsensitiveStringComparison cmp;
+			
 			if (!itsConstraints[0].empty())
 				any_found =
-					SEStringComparison((*it)->GetArtist(), itsConstraints[0], CaseSensitive)
-				||	SEStringComparison((*it)->GetTitle(), itsConstraints[0], CaseSensitive)
-				||	SEStringComparison((*it)->GetAlbum(), itsConstraints[0], CaseSensitive)
-				||	SEStringComparison((*it)->GetName(), itsConstraints[0], CaseSensitive)
-				||	SEStringComparison((*it)->GetComposer(), itsConstraints[0], CaseSensitive)
-				||	SEStringComparison((*it)->GetPerformer(), itsConstraints[0], CaseSensitive)
-				||	SEStringComparison((*it)->GetGenre(), itsConstraints[0], CaseSensitive)
-				||	SEStringComparison((*it)->GetDate(), itsConstraints[0], CaseSensitive)
-				||	SEStringComparison((*it)->GetComment(), itsConstraints[0], CaseSensitive);
+					!cmp((*it)->GetArtist(), itsConstraints[0])
+				||	!cmp((*it)->GetTitle(), itsConstraints[0])
+				||	!cmp((*it)->GetAlbum(), itsConstraints[0])
+				||	!cmp((*it)->GetName(), itsConstraints[0])
+				||	!cmp((*it)->GetComposer(), itsConstraints[0])
+				||	!cmp((*it)->GetPerformer(), itsConstraints[0])
+				||	!cmp((*it)->GetGenre(), itsConstraints[0])
+				||	!cmp((*it)->GetDate(), itsConstraints[0])
+				||	!cmp((*it)->GetComment(), itsConstraints[0]);
 			
 			if (found && !itsConstraints[1].empty())
-				found = SEStringComparison((*it)->GetArtist(), itsConstraints[1], CaseSensitive);
+				found = !cmp((*it)->GetArtist(), itsConstraints[1]);
 			if (found && !itsConstraints[2].empty())
-				found = SEStringComparison((*it)->GetTitle(), itsConstraints[2], CaseSensitive);
+				found = !cmp((*it)->GetTitle(), itsConstraints[2]);
 			if (found && !itsConstraints[3].empty())
-				found = SEStringComparison((*it)->GetAlbum(), itsConstraints[3], CaseSensitive);
+				found = !cmp((*it)->GetAlbum(), itsConstraints[3]);
 			if (found && !itsConstraints[4].empty())
-				found = SEStringComparison((*it)->GetName(), itsConstraints[4], CaseSensitive);
+				found = !cmp((*it)->GetName(), itsConstraints[4]);
 			if (found && !itsConstraints[5].empty())
-				found = SEStringComparison((*it)->GetComposer(), itsConstraints[5], CaseSensitive);
+				found = !cmp((*it)->GetComposer(), itsConstraints[5]);
 			if (found && !itsConstraints[6].empty())
-				found = SEStringComparison((*it)->GetPerformer(), itsConstraints[6], CaseSensitive);
+				found = !cmp((*it)->GetPerformer(), itsConstraints[6]);
 			if (found && !itsConstraints[7].empty())
-				found = SEStringComparison((*it)->GetGenre(), itsConstraints[7], CaseSensitive);
+				found = !cmp((*it)->GetGenre(), itsConstraints[7]);
 			if (found && !itsConstraints[8].empty())
-				found = SEStringComparison((*it)->GetDate(), itsConstraints[8], CaseSensitive);
+				found = !cmp((*it)->GetDate(), itsConstraints[8]);
 			if (found && !itsConstraints[9].empty())
-				found = SEStringComparison((*it)->GetComment(), itsConstraints[9], CaseSensitive);
+				found = !cmp((*it)->GetComment(), itsConstraints[9]);
 		}
 		
 		if (found && any_found)
 		{
-			Song *ss = Config.search_in_db ? *it : new Song(**it);
+			MPD::Song *ss = Config.search_in_db ? *it : new MPD::Song(**it);
 			w->AddOption(std::make_pair(static_cast<Buffer *>(0), ss));
 			list[it-list.begin()] = 0;
 		}
@@ -445,12 +491,7 @@ void SearchEngine::Search()
 		any_found = 1;
 	}
 	if (Config.search_in_db) // free song list only if it's database
-		FreeSongList(list);
-}
-
-bool SearchEngine::SEStringComparison(const std::string &a, const std::string &b, bool case_sensitive)
-{
-	return case_sensitive ? a == b : !CaseInsensitiveStringComparison()(a, b);
+		MPD::FreeSongList(list);
 }
 
 std::string SearchEngine::SearchEngineOptionToString(const std::pair<Buffer *, MPD::Song *> &pair, void *)
