@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008-2009 by Andrzej Rybczak                            *
+ *   Copyright (C) 2008-2010 by Andrzej Rybczak                            *
  *   electricityispower@gmail.com                                          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -35,6 +35,7 @@ class SearchEngine : public Screen< Menu< std::pair<Buffer *, MPD::Song *> > >
 		virtual void EnterPressed();
 		virtual void SpacePressed();
 		virtual void MouseButtonPressed(MEVENT);
+		virtual bool isTabbable() { return true; }
 		
 		virtual MPD::Song *CurrentSong();
 		
@@ -52,25 +53,25 @@ class SearchEngine : public Screen< Menu< std::pair<Buffer *, MPD::Song *> > >
 		static size_t SearchButton;
 		static size_t ResetButton;
 		
-		static const char *NormalMode;
-		static const char *StrictMode;
-		
 	protected:
 		virtual void Init();
 		
 	private:
 		void Prepare();
 		void Search();
+		void Reset();
+		
+		const char **SearchMode;
 		
 		static std::string SearchEngineOptionToString(const std::pair<Buffer *, MPD::Song *> &, void *);
-		static bool SEStringComparison(const std::string &a, const std::string &b, bool case_sensitive);
+		
+		static const char *SearchModes[];
 		
 		static const size_t ConstraintsNumber = 10;
 		static const char *ConstraintsNames[];
 		std::string itsConstraints[ConstraintsNumber];
 		
 		static bool MatchToPattern;
-		static int CaseSensitive;
 };
 
 extern SearchEngine *mySearcher;
