@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008-2009 by Andrzej Rybczak                            *
+ *   Copyright (C) 2008-2010 by Andrzej Rybczak                            *
  *   electricityispower@gmail.com                                          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -24,8 +24,9 @@
 #include <limits>
 #include <vector>
 
+#include <mpd/client.h>
+
 #include "home.h"
-#include "libmpdclient.h"
 #include "ncmpcpp.h"
 
 #ifdef WIN32
@@ -37,6 +38,8 @@
 const std::string config_dir = home_path + HOME_FOLDER;
 const int null_key = std::numeric_limits<int>::max();
 
+class BasicScreen; // forward declaration for screens sequence
+
 struct Column
 {
 	Column() : right_alignment(0), display_empty_tag(1) { }
@@ -49,8 +52,11 @@ struct Column
 	bool display_empty_tag;
 };
 
-struct ncmpcpp_keys
+struct NcmpcppKeys
 {
+	void SetDefaults();
+	void Read();
+	
 	int Up[2];
 	int Down[2];
 	int PageUp[2];
@@ -73,6 +79,7 @@ struct ncmpcpp_keys
 	int Outputs[2];
 	int Visualizer[2];
 	int Clock[2];
+	int ServerInfo[2];
 	int Stop[2];
 	int Pause[2];
 	int Next[2];
@@ -83,6 +90,7 @@ struct ncmpcpp_keys
 	int ToggleRandom[2];
 	int ToggleSingle[2];
 	int ToggleConsume[2];
+	int ToggleReplayGainMode[2];
 	int ToggleSpaceMode[2];
 	int ToggleAddMode[2];
 	int ToggleMouse[2];
@@ -114,6 +122,7 @@ struct ncmpcpp_keys
 	int SavePlaylist[2];
 	int GoToNowPlaying[2];
 	int GoToContainingDir[2];
+	int GoToMediaLibrary[2];
 	int ToggleAutoCenter[2];
 	int ToggleDisplayMode[2];
 	int ToggleInterface[2];
@@ -123,11 +132,15 @@ struct ncmpcpp_keys
 	int Quit[2];
 };
 
-struct ncmpcpp_config
+struct NcmpcppConfig
 {
+	void SetDefaults();
+	void Read();
+	
 	std::string mpd_host;
 	std::string mpd_music_dir;
 	std::string visualizer_fifo_path;
+	std::string visualizer_output_name;
 	std::string empty_tag;
 	std::string song_list_columns_format;
 	std::string song_list_format;
@@ -170,8 +183,9 @@ struct ncmpcpp_config
 	Border window_border;
 	Border active_window_border;
 	
-	mpd_TagItems media_lib_primary_tag;
+	mpd_tag_type media_lib_primary_tag;
 	
+	bool enable_idle_notifications;
 	bool colors_enabled;
 	bool fancy_scrolling;
 	bool playlist_show_remaining_time;
@@ -182,6 +196,8 @@ struct ncmpcpp_config
 	bool header_visibility;
 	bool header_text_scrolling;
 	bool statusbar_visibility;
+	bool centered_cursor;
+	bool screen_switcher_previous;
 	bool autocenter_mode;
 	bool wrapped_search;
 	bool space_selects;
@@ -204,8 +220,14 @@ struct ncmpcpp_config
 	bool allow_physical_directories_deletion;
 	bool ask_before_clearing_main_playlist;
 	bool mouse_support;
+	bool mouse_list_scroll_whole_page;
 	bool new_design;
 	bool visualizer_use_wave;
+	bool browser_sort_by_mtime;
+	bool tag_editor_extended_numeration;
+	bool media_library_display_date;
+	bool media_library_disable_two_column_mode;
+	bool discard_colors_if_item_is_selected;
 	
 	int mpd_port;
 	int mpd_connection_timeout;
@@ -217,17 +239,23 @@ struct ncmpcpp_config
 	int regex_type;
 	
 	unsigned lines_scrolled;
+	unsigned search_engine_default_search_mode;
+	unsigned visualizer_sync_interval;
+	
+	size_t selected_item_suffix_length;
+	size_t now_playing_suffix_length;
+	
+	BasicScreen *startup_screen;
+	std::list<BasicScreen *> screens_seq;
 };
 
-extern ncmpcpp_config Config;
-extern ncmpcpp_keys Key;
+extern NcmpcppKeys Key;
+extern NcmpcppConfig Config;
 
 void CreateConfigDir();
 void SetWindowsDimensions(size_t &header_height, size_t &footer_start_y, size_t &footer_height);
-void DefaultKeys(ncmpcpp_keys &);
-void DefaultConfiguration(ncmpcpp_config &);
-void ReadKeys(ncmpcpp_keys &);
-void ReadConfiguration(ncmpcpp_config &);
+void ReadKeys(NcmpcppKeys &);
+void ReadConfiguration(NcmpcppConfig &);
 
 #endif
 
