@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008-2009 by Andrzej Rybczak                            *
+ *   Copyright (C) 2008-2010 by Andrzej Rybczak                            *
  *   electricityispower@gmail.com                                          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -19,6 +19,8 @@
  ***************************************************************************/
 
 #include <cstring>
+#include <sys/time.h>
+
 #include <iostream>
 #include <stdexcept>
 
@@ -28,21 +30,28 @@
 #include "helpers.h"
 #include "lyrics.h"
 #include "media_library.h"
+#include "outputs.h"
 #include "playlist.h"
 #include "playlist_editor.h"
 #include "search_engine.h"
+#include "sel_items_adder.h"
 #include "settings.h"
 #include "status.h"
 #include "tag_editor.h"
 
-using namespace Global;
-using namespace MPD;
-
-std::string Global::VolumeState;
+using Global::myScreen;
+using Global::wFooter;
+using Global::Timer;
+using Global::wHeader;
+using Global::VolumeState;
 
 bool Global::UpdateStatusImmediately = 0;
 bool Global::RedrawStatusbar = 0;
 
+std::string Global::VolumeState;
+
+timeval Global::Timer;
+
 namespace
 {
 	time_t time_of_statusbar_lock;
@@ -51,8 +60,6 @@ namespace
 	bool block_statusbar_update = 0;
 	bool block_progressbar_update = 0;
 	bool allow_statusbar_unlock = 1;
-	
-	int local_elapsed;
 }
 
 #ifndef USE_PDCURSES
@@ -63,6 +70,11 @@ void WindowTitle(const std::string &status)
 }
 #endif // !USE_PDCURSES
 
+void StatusbarMPDCallback()
+{
+	Mpd.OrderDataFetching();
+}
+
 void StatusbarGetStringHelper(const std::wstring &)
 {
 	TraceMpdStatus();
@@ -109,25 +121,44 @@ void UnlockStatusbar()
 			block_progressbar_update = 0;
 	}
 	if (!Mpd.isPlaying())
-		Statusbar() << wclrtoeol;
+	{
+		if (Config.new_design)
+			DrawProgressbar(Mpd.GetElapsedTime(), Mpd.GetTotalTime());
+		else
+			Statusbar() << wclrtoeol;
+		wFooter->Refresh();
+	}
 }
 
 void TraceMpdStatus()
 {
-	static timeval past, now;
+	static timeval past = { 0, 0 };
 	
-	gettimeofday(&now, 0);
-	if ((Mpd.Connected() && now.tv_sec > past.tv_sec) || UpdateStatusImmediately)
+	gettimeofday(&Global::Timer, 0);
+	if (Mpd.Connected() && (Mpd.SupportsIdle() || Timer.tv_sec > past.tv_sec || Global::UpdateStatusImmediately))
 	{
 		Mpd.UpdateStatus();
-		BlockItemListUpdate = 0;
+		Global::BlockItemListUpdate = 0;
 		Playlist::BlockUpdate = 0;
-		UpdateStatusImmediately = 0;
-		gettimeofday(&past, 0);
+		Global::UpdateStatusImmediately = 0;
+		if (!Mpd.SupportsIdle())
+		{
+			gettimeofday(&past, 0);
+		}
+		else if (Config.display_bitrate && Global::Timer.tv_sec > past.tv_sec && Mpd.isPlaying())
+		{
+			// ncmpcpp doesn't fetch status constantly if mpd supports
+			// idle mode so current song's bitrate is never updated.
+			// we need to force ncmpcpp to fetch it.
+			Mpd.OrderDataFetching();
+			gettimeofday(&past, 0);
+		}
 	}
 	
+	myScreen->Update();
+	
 	if (myScreen->ActiveWindow() == myPlaylist->Items
-	&&  now.tv_sec == myPlaylist->Timer()+Config.playlist_disable_highlight_delay
+	&&  Timer.tv_sec == myPlaylist->Timer()+Config.playlist_disable_highlight_delay
 	&&  myPlaylist->Items->isHighlighted()
 	&&  Config.playlist_disable_highlight_delay)
 	{
@@ -137,7 +168,7 @@ void TraceMpdStatus()
 	
 	if (lock_statusbar_delay > 0)
 	{
-		if (now.tv_sec >= time_of_statusbar_lock+lock_statusbar_delay)
+		if (Timer.tv_sec >= time_of_statusbar_lock+lock_statusbar_delay)
 		{
 			lock_statusbar_delay = -1;
 			
@@ -146,10 +177,10 @@ void TraceMpdStatus()
 			else
 				block_progressbar_update = !allow_statusbar_unlock;
 			
-			if (Mpd.GetState() != psPlay && !block_statusbar_update && !block_progressbar_update)
+			if (Mpd.GetState() != MPD::psPlay && !block_statusbar_update && !block_progressbar_update)
 			{
 				if (Config.new_design)
-					DrawProgressbar(local_elapsed, Mpd.GetTotalTime());
+					DrawProgressbar(Mpd.GetElapsedTime(), Mpd.GetTotalTime());
 				else
 					Statusbar() << wclrtoeol;
 				wFooter->Refresh();
@@ -158,23 +189,22 @@ void TraceMpdStatus()
 	}
 }
 
-void NcmpcppErrorCallback(Connection *, int errorid, const char *msg, void *)
+void NcmpcppErrorCallback(MPD::Connection *, int errorid, const char *msg, void *)
 {
-	if (errorid == MPD_ACK_ERROR_PERMISSION)
+	if (errorid == MPD_SERVER_ERROR_PERMISSION)
 	{
 		wFooter->SetGetStringHelper(0);
 		Statusbar() << "Password: ";
-		std::string password = wFooter->GetString(-1, 0, 1);
-		Mpd.SetPassword(password);
-		Mpd.SendPassword();
-		Mpd.UpdateStatus();
+		Mpd.SetPassword(wFooter->GetString(-1, 0, 1));
+		if (Mpd.SendPassword())
+			ShowMessage("Password accepted!");
 		wFooter->SetGetStringHelper(StatusbarGetStringHelper);
 	}
 	else
 		ShowMessage("%s", msg);
 }
 
-void NcmpcppStatusChanged(Connection *, StatusChanges changed, void *)
+void NcmpcppStatusChanged(MPD::Connection *, MPD::StatusChanges changed, void *)
 {
 	static size_t playing_song_scroll_begin = 0;
 	static size_t first_line_scroll_begin = 0;
@@ -193,13 +223,12 @@ void NcmpcppStatusChanged(Connection *, StatusChanges changed, void *)
 	{
 		if (!Playlist::BlockUpdate)
 		{
-			np = Mpd.GetCurrentSong();
-			if (Mpd.isPlaying())
+			if (!(np = Mpd.GetCurrentSong()).Empty())
 				WindowTitle(utf_to_locale_cpy(np.toString(Config.song_window_title_format)));
 			
 			bool was_filtered = myPlaylist->Items->isFiltered();
 			myPlaylist->Items->ShowAll();
-			SongList list;
+			MPD::SongList list;
 			
 			size_t playlist_length = Mpd.GetPlaylistLength();
 			if (playlist_length < myPlaylist->Items->Size())
@@ -207,7 +236,7 @@ void NcmpcppStatusChanged(Connection *, StatusChanges changed, void *)
 			
 			Mpd.GetPlaylistChanges(Mpd.GetOldPlaylistID(), list);
 			myPlaylist->Items->Reserve(playlist_length);
-			for (SongList::const_iterator it = list.begin(); it != list.end(); ++it)
+			for (MPD::SongList::const_iterator it = list.begin(); it != list.end(); ++it)
 			{
 				int pos = (*it)->GetPosition();
 				if (pos < int(myPlaylist->Items->Size()))
@@ -238,11 +267,11 @@ void NcmpcppStatusChanged(Connection *, StatusChanges changed, void *)
 		if (myPlaylist->Items->Empty())
 		{
 			myPlaylist->Items->Reset();
-			myPlaylist->Items->Window::Clear(0);
+			myPlaylist->Items->Window::Clear();
 			ShowMessage("Cleared playlist!");
 		}
 		
-		if (!BlockItemListUpdate)
+		if (!Global::BlockItemListUpdate)
 		{
 			if (myScreen == myBrowser)
 			{
@@ -265,50 +294,55 @@ void NcmpcppStatusChanged(Connection *, StatusChanges changed, void *)
 	if (changed.Database)
 	{
 		if (myBrowser->Main())
-			myBrowser->GetDirectory(myBrowser->CurrentDir());
+		{
+			if (myScreen == myBrowser)
+				myBrowser->GetDirectory(myBrowser->CurrentDir());
+			else
+				myBrowser->Main()->Clear();
+		}
 #		ifdef HAVE_TAGLIB_H
 		if (myTagEditor->Main())
 		{
-			myTagEditor->Albums->Clear(0);
-			myTagEditor->Dirs->Clear(0);
+			myTagEditor->Albums->Clear();
+			myTagEditor->Dirs->Clear();
 		}
 #		endif // HAVE_TAGLIB_H
 		if (myLibrary->Main())
 		{
 			if (myLibrary->Columns() == 2)
-			{
 				myLibrary->Albums->Clear();
-				myLibrary->Songs->Clear(0);
-			}
 			else
-				myLibrary->Artists->Clear(0);
+				myLibrary->Artists->Clear();
 		}
 		if (myPlaylistEditor->Main())
-			myPlaylistEditor->Content->Clear(0);
+			myPlaylistEditor->Content->Clear();
+		changed.DBUpdating = 1;
 	}
 	if (changed.PlayerState)
 	{
 		switch (Mpd.GetState())
 		{
-			case psUnknown:
+			case MPD::psUnknown:
 			{
 				player_state = "[unknown]";
 				break;
 			}
-			case psPlay:
+			case MPD::psPlay:
 			{
-				WindowTitle(utf_to_locale_cpy(np.toString(Config.song_window_title_format)));
+				if (!np.Empty())
+					WindowTitle(utf_to_locale_cpy(np.toString(Config.song_window_title_format)));
 				player_state = Config.new_design ? "[playing]" : "Playing: ";
 				Playlist::ReloadRemaining = 1;
-				changed.ElapsedTime = 1;
+				if (Mpd.GetOldState() == MPD::psStop) // show track info in status immediately
+					changed.ElapsedTime = 1;
 				break;
 			}
-			case psPause:
+			case MPD::psPause:
 			{
 				player_state = Config.new_design ? "[paused] " : "[Paused] ";
 				break;
 			}
-			case psStop:
+			case MPD::psStop:
 			{
 				WindowTitle("ncmpc++ ver. "VERSION);
 				if (!block_progressbar_update)
@@ -333,70 +367,55 @@ void NcmpcppStatusChanged(Connection *, StatusChanges changed, void *)
 		}
 		if (Config.new_design)
 		{
-			if (!myPlaylist->isPlaying())
-			{
-				*wHeader << XY(0, 1) << fmtBold << player_state << fmtBoldEnd;
-				wHeader->Refresh();
-			}
+			*wHeader << XY(0, 1) << fmtBold << player_state << fmtBoldEnd;
+			wHeader->Refresh();
+		}
+		else if (!block_statusbar_update && Config.statusbar_visibility)
+		{
+			*wFooter << XY(0, 1);
+			if (player_state.empty())
+				*wFooter << wclrtoeol;
+			else
+				*wFooter << fmtBold << player_state << fmtBoldEnd;
 		}
-		else if (!block_statusbar_update && Config.statusbar_visibility && player_state.empty())
-			*wFooter << XY(0, 1) << wclrtoeol;
 	}
 	if (changed.SongID)
 	{
 		if (myPlaylist->isPlaying())
 		{
-			np = Mpd.GetCurrentSong();
-			
 			if (!Config.execute_on_song_change.empty())
 				system(Config.execute_on_song_change.c_str());
-			if (Mpd.isPlaying())
+			
+			if (Mpd.isPlaying() && !(np = Mpd.GetCurrentSong()).Empty())
 				WindowTitle(utf_to_locale_cpy(np.toString(Config.song_window_title_format)));
+			
 			if (Config.autocenter_mode && !myPlaylist->Items->isFiltered())
 				myPlaylist->Items->Highlight(myPlaylist->NowPlaying);
 			
-			if (!Mpd.GetElapsedTime())
-				mvwhline(wFooter->Raw(), 0, 0, 0, wFooter->GetWidth());
-			
-			if (Config.now_playing_lyrics && !Mpd.GetSingle() && myScreen == myLyrics && myOldScreen == myPlaylist)
+			if (Config.now_playing_lyrics && !Mpd.GetSingle() && myScreen == myLyrics && Global::myOldScreen == myPlaylist)
 				Lyrics::Reload = 1;
 		}
 		Playlist::ReloadRemaining = 1;
-		
 		playing_song_scroll_begin = 0;
 		first_line_scroll_begin = 0;
 		second_line_scroll_begin = 0;
 	}
-	static time_t now, past = 0;
-	time(&now);
-	if (((now > past || changed.SongID) && Mpd.isPlaying()) || RedrawStatusbar)
+	if (changed.ElapsedTime || changed.SongID || Global::RedrawStatusbar)
 	{
-		time(&past);
-		if (np.Empty())
-		{
-			np = Mpd.GetCurrentSong();
+		if (np.Empty() && !(np = Mpd.GetCurrentSong()).Empty())
 			WindowTitle(utf_to_locale_cpy(np.toString(Config.song_window_title_format)));
-		}
 		if (!np.Empty() && Mpd.isPlaying())
 		{
-			changed.ElapsedTime = 1;
-			
-			int mpd_elapsed = Mpd.GetElapsedTime();
-			if (local_elapsed < mpd_elapsed-2 || local_elapsed+1 > mpd_elapsed)
-				local_elapsed = mpd_elapsed;
-			else if (Mpd.GetState() == psPlay && !RedrawStatusbar)
-				++local_elapsed;
-			
 			std::string tracklength;
 			if (Config.new_design)
 			{
 				if (Config.display_remaining_time)
 				{
 					tracklength = "-";
-					tracklength += Song::ShowTime(Mpd.GetTotalTime()-local_elapsed);
+					tracklength += MPD::Song::ShowTime(Mpd.GetTotalTime()-Mpd.GetElapsedTime());
 				}
 				else
-					tracklength = Song::ShowTime(local_elapsed);
+					tracklength = MPD::Song::ShowTime(Mpd.GetElapsedTime());
 				if (Mpd.GetTotalTime())
 				{
 					tracklength += "/";
@@ -422,7 +441,7 @@ void NcmpcppStatusChanged(Connection *, StatusChanges changed, void *)
 				size_t second_margin = (std::max(player_state.length(), size_t(8))+1)*2;
 				size_t second_start = second_len < COLS-second_margin ? (COLS-second_len)/2 : player_state.length()+1;
 				
-				if (!SeekingInProgress)
+				if (!Global::SeekingInProgress)
 					*wHeader << XY(0, 0) << wclrtoeol << tracklength;
 				*wHeader << XY(first_start, 0);
 				first.Write(*wHeader, first_line_scroll_begin, COLS-tracklength.length()-VolumeState.length()-1, U(" ** "));
@@ -449,17 +468,17 @@ void NcmpcppStatusChanged(Connection *, StatusChanges changed, void *)
 					if (Config.display_remaining_time)
 					{
 						tracklength += "-";
-						tracklength += Song::ShowTime(Mpd.GetTotalTime()-local_elapsed);
+						tracklength += MPD::Song::ShowTime(Mpd.GetTotalTime()-Mpd.GetElapsedTime());
 					}
 					else
-						tracklength += Song::ShowTime(local_elapsed);
+						tracklength += MPD::Song::ShowTime(Mpd.GetElapsedTime());
 					tracklength += "/";
 					tracklength += MPD::Song::ShowTime(Mpd.GetTotalTime());
 					tracklength += "]";
 				}
 				else
 				{
-					tracklength += Song::ShowTime(local_elapsed);
+					tracklength += MPD::Song::ShowTime(Mpd.GetElapsedTime());
 					tracklength += "]";
 				}
 				basic_buffer<my_char_t> np_song;
@@ -469,8 +488,8 @@ void NcmpcppStatusChanged(Connection *, StatusChanges changed, void *)
 				*wFooter << fmtBold << XY(wFooter->GetWidth()-tracklength.length(), 1) << tracklength;
 			}
 			if (!block_progressbar_update)
-				DrawProgressbar(local_elapsed, Mpd.GetTotalTime());
-			RedrawStatusbar = 0;
+				DrawProgressbar(Mpd.GetElapsedTime(), Mpd.GetTotalTime());
+			Global::RedrawStatusbar = 0;
 		}
 		else
 		{
@@ -514,8 +533,16 @@ void NcmpcppStatusChanged(Connection *, StatusChanges changed, void *)
 	}
 	if (changed.DBUpdating)
 	{
-		mpd_db_updating = Mpd.GetDBIsUpdating() ? 'U' : 0;
-		ShowMessage(!mpd_db_updating ? "Database update finished!" : "Database update started!");
+		// mpd-0.{14,15} doesn't support idle notification that dbupdate had
+		// finished and nothing changed, so we need to switch it off for them.
+		if (!Mpd.SupportsIdle() || Mpd.Version() > 15)
+			mpd_db_updating = Mpd.GetDBIsUpdating() ? 'U' : 0;
+		ShowMessage(Mpd.GetDBIsUpdating() ? "Database update started!" : "Database update finished!");
+		if (changed.Database && myScreen == mySelectedItemsAdder)
+		{
+			myScreen->SwitchTo(); // switch to previous screen
+			ShowMessage("Database has changed, you need to select your item(s) once again!");
+		}
 	}
 	if (changed.StatusFlags && (Config.header_visibility || Config.new_design))
 	{
@@ -589,9 +616,15 @@ void NcmpcppStatusChanged(Connection *, StatusChanges changed, void *)
 		*wHeader << clEnd;
 		wHeader->Refresh();
 	}
+	if (changed.Outputs)
+	{
+#		ifdef ENABLE_OUTPUTS
+		myOutputs->FetchList();
+#		endif // ENABLE_OUTPUTS
+	}
 	*wFooter << fmtBoldEnd;
 	wFooter->GotoXY(sx, sy);
-	if (changed.PlayerState || (changed.ElapsedTime && (!Config.new_design || Mpd.GetState() == psPlay)))
+	if (changed.PlayerState || (changed.ElapsedTime && (!Config.new_design || Mpd.GetState() == MPD::psPlay)))
 		wFooter->Refresh();
 	if (changed.Playlist || changed.Database || changed.PlayerState || changed.SongID)
 		myScreen->RefreshWindow();
@@ -621,7 +654,7 @@ void DrawProgressbar(unsigned elapsed, unsigned time)
 
 void ShowMessage(const char *format, ...)
 {
-	if (MessagesAllowed && allow_statusbar_unlock)
+	if (Global::MessagesAllowed && allow_statusbar_unlock)
 	{
 		time(&time_of_statusbar_lock);
 		lock_statusbar_delay = Config.message_delay_time;
