@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008-2009 by Andrzej Rybczak                            *
+ *   Copyright (C) 2008-2010 by Andrzej Rybczak                            *
  *   electricityispower@gmail.com                                          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -32,16 +32,15 @@
 #include "vorbisfile.h"
 #include "flacfile.h"
 
+#include "browser.h"
 #include "charset.h"
 #include "display.h"
 #include "global.h"
+#include "info.h"
 #include "playlist.h"
 
 using Global::MainHeight;
 using Global::MainStartY;
-using Global::myOldScreen;
-using Global::myScreen;
-using Global::wFooter;
 
 TagEditor *myTagEditor = new TagEditor;
 
@@ -67,15 +66,15 @@ void TagEditor::Init()
 	
 	Albums = new Menu<string_pair>(0, MainStartY, LeftColumnWidth, MainHeight, "Albums", Config.main_color, brNone);
 	Albums->HighlightColor(Config.active_column_color);
-	Albums->SetTimeout(ncmpcpp_window_timeout);
 	Albums->CyclicScrolling(Config.use_cyclic_scrolling);
+	Albums->CenteredCursor(Config.centered_cursor);
 	Albums->SetItemDisplayer(Display::Pairs);
 	Albums->SetGetStringFunction(StringPairToString);
 	
 	Dirs = new Menu<string_pair>(0, MainStartY, LeftColumnWidth, MainHeight, "Directories", Config.main_color, brNone);
 	Dirs->HighlightColor(Config.active_column_color);
-	Dirs->SetTimeout(ncmpcpp_window_timeout);
 	Dirs->CyclicScrolling(Config.use_cyclic_scrolling);
+	Dirs->CenteredCursor(Config.centered_cursor);
 	Dirs->SetItemDisplayer(Display::Pairs);
 	Dirs->SetGetStringFunction(StringPairToString);
 	
@@ -83,20 +82,12 @@ void TagEditor::Init()
 	
 	TagTypes = new Menu<std::string>(MiddleColumnStartX, MainStartY, MiddleColumnWidth, MainHeight, "Tag types", Config.main_color, brNone);
 	TagTypes->HighlightColor(Config.main_highlight_color);
-	TagTypes->SetTimeout(ncmpcpp_window_timeout);
 	TagTypes->CyclicScrolling(Config.use_cyclic_scrolling);
+	TagTypes->CenteredCursor(Config.centered_cursor);
 	TagTypes->SetItemDisplayer(Display::Generic);
 	
-	TagTypes->AddOption("Title");
-	TagTypes->AddOption("Artist");
-	TagTypes->AddOption("Album");
-	TagTypes->AddOption("Year");
-	TagTypes->AddOption("Track");
-	TagTypes->AddOption("Genre");
-	TagTypes->AddOption("Composer");
-	TagTypes->AddOption("Performer");
-	TagTypes->AddOption("Disc");
-	TagTypes->AddOption("Comment");
+	for (const Info::Metadata *m = Info::Tags; m->Name; ++m)
+		TagTypes->AddOption(m->Name);
 	TagTypes->AddSeparator();
 	TagTypes->AddOption("Filename");
 	TagTypes->AddSeparator();
@@ -110,8 +101,8 @@ void TagEditor::Init()
 	
 	Tags = new Menu<MPD::Song>(RightColumnStartX, MainStartY, RightColumnWidth, MainHeight, "Tags", Config.main_color, brNone);
 	Tags->HighlightColor(Config.main_highlight_color);
-	Tags->SetTimeout(ncmpcpp_window_timeout);
 	Tags->CyclicScrolling(Config.use_cyclic_scrolling);
+	Tags->CenteredCursor(Config.centered_cursor);
 	Tags->SetSelectPrefix(&Config.selected_item_prefix);
 	Tags->SetSelectSuffix(&Config.selected_item_suffix);
 	Tags->SetItemDisplayer(Display::Tags);
@@ -120,8 +111,8 @@ void TagEditor::Init()
 	Tags->SetGetStringFunctionUserData(TagTypes);
 	
 	FParserDialog = new Menu<std::string>((COLS-FParserDialogWidth)/2, (MainHeight-FParserDialogHeight)/2+MainStartY, FParserDialogWidth, FParserDialogHeight, "", Config.main_color, Config.window_border);
-	FParserDialog->SetTimeout(ncmpcpp_window_timeout);
 	FParserDialog->CyclicScrolling(Config.use_cyclic_scrolling);
+	FParserDialog->CenteredCursor(Config.centered_cursor);
 	FParserDialog->SetItemDisplayer(Display::Generic);
 	FParserDialog->AddOption("Get tags from filename");
 	FParserDialog->AddOption("Rename files");
@@ -129,15 +120,13 @@ void TagEditor::Init()
 	FParserDialog->AddOption("Cancel");
 	
 	FParser = new Menu<std::string>((COLS-FParserWidth)/2, (MainHeight-FParserHeight)/2+MainStartY, FParserWidthOne, FParserHeight, "_", Config.main_color, Config.active_window_border);
-	FParser->SetTimeout(ncmpcpp_window_timeout);
 	FParser->CyclicScrolling(Config.use_cyclic_scrolling);
+	FParser->CenteredCursor(Config.centered_cursor);
 	FParser->SetItemDisplayer(Display::Generic);
 	
 	FParserLegend = new Scrollpad((COLS-FParserWidth)/2+FParserWidthOne, (MainHeight-FParserHeight)/2+MainStartY, FParserWidthTwo, FParserHeight, "Legend", Config.main_color, Config.window_border);
-	FParserLegend->SetTimeout(ncmpcpp_window_timeout);
 	
 	FParserPreview = new Scrollpad((COLS-FParserWidth)/2+FParserWidthOne, (MainHeight-FParserHeight)/2+MainStartY, FParserWidthTwo, FParserHeight, "Preview", Config.main_color, Config.window_border);
-	FParserPreview->SetTimeout(ncmpcpp_window_timeout);
 	
 	w = LeftColumn;
 	isInitialized = 1;
@@ -195,6 +184,8 @@ std::basic_string<my_char_t> TagEditor::Title()
 
 void TagEditor::SwitchTo()
 {
+	using Global::myScreen;
+	
 	if (myScreen == this)
 		return;
 	
@@ -204,6 +195,8 @@ void TagEditor::SwitchTo()
 	if (hasToBeResized)
 		Resize();
 	
+	if (myScreen != this && myScreen->isTabbable())
+		Global::myPrevScreen = myScreen;
 	myScreen = this;
 	Global::RedrawHeader = 1;
 	Refresh();
@@ -217,10 +210,15 @@ void TagEditor::Refresh()
 	mvvline(MainStartY, RightColumnStartX-1, 0, MainHeight);
 	Tags->Display();
 	
-	if (w == FParser)
-		FParserHelper->Display();
-	else if (w == FParserHelper)
+	if (w == FParserDialog)
+	{
+		FParserDialog->Display();
+	}
+	else if (w == FParser || w == FParserHelper)
+	{
 		FParser->Display();
+		FParserHelper->Display();
+	}
 }
 
 void TagEditor::Update()
@@ -234,12 +232,12 @@ void TagEditor::Update()
 		{
 			*Albums << XY(0, 0) << "Fetching albums...";
 			Albums->Window::Refresh();
-			Mpd.GetAlbums("", list);
+			Mpd.GetList(list, MPD_TAG_ALBUM);
 			for (MPD::TagList::const_iterator it = list.begin(); it != list.end(); ++it)
 			{
 				MPD::SongList l;
 				Mpd.StartSearch(1);
-				Mpd.AddSearch(MPD_TAG_ITEM_ALBUM, *it);
+				Mpd.AddSearch(MPD_TAG_ALBUM, *it);
 				Mpd.CommitSearch(l);
 				if (!l.empty())
 				{
@@ -288,7 +286,7 @@ void TagEditor::Update()
 		if (Config.albums_in_tag_editor)
 		{
 			Mpd.StartSearch(1);
-			Mpd.AddSearch(MPD_TAG_ITEM_ALBUM, Albums->Current().second);
+			Mpd.AddSearch(MPD_TAG_ALBUM, Albums->Current().second);
 			Mpd.CommitSearch(list);
 			sort(list.begin(), list.end(), CaseInsensitiveSorting());
 			for (MPD::SongList::iterator it = list.begin(); it != list.end(); ++it)
@@ -317,11 +315,16 @@ void TagEditor::Update()
 		Tags->Refresh();
 	}
 	else if (TagTypes->Choice() >= 13)
+	{
 		Tags->Window::Clear();
+		Tags->Window::Refresh();
+	}
 }
 
 void TagEditor::EnterPressed()
 {
+	using Global::wFooter;
+	
 	if (w == Dirs)
 	{
 		MPD::TagList test;
@@ -330,7 +333,7 @@ void TagEditor::EnterPressed()
 		{
 			itsHighlightedDir = itsBrowsedDir;
 			itsBrowsedDir = LeftColumn->Current().second;
-			LeftColumn->Clear(0);
+			LeftColumn->Clear();
 			LeftColumn->Reset();
 		}
 		else
@@ -351,6 +354,7 @@ void TagEditor::EnterPressed()
 		
 		FParserLegend->Clear();
 		*FParserLegend << "%a - artist\n";
+		*FParserLegend << "%A - album artist\n";
 		*FParserLegend << "%t - title\n";
 		*FParserLegend << "%b - album\n";
 		*FParserLegend << "%y - year\n";
@@ -507,76 +511,65 @@ void TagEditor::EnterPressed()
 		for (size_t i = 0; i < Tags->Size(); ++i)
 			EditedSongs.push_back(&(*Tags)[i]);
 	
-	MPD::Song::GetFunction get = 0;
-	MPD::Song::SetFunction set = 0;
-	
 	size_t id = TagTypes->RealChoice();
-	switch (id)
+	
+	if (w == TagTypes && id == 5)
 	{
-		case 0:
-			get = &MPD::Song::GetTitle;
-			set = &MPD::Song::SetTitle;
-			break;
-		case 1:
-			get = &MPD::Song::GetArtist;
-			set = &MPD::Song::SetArtist;
-			break;
-		case 2:
-			get = &MPD::Song::GetAlbum;
-			set = &MPD::Song::SetAlbum;
-			break;
-		case 3:
-			get = &MPD::Song::GetDate;
-			set = &MPD::Song::SetDate;
-			break;
-		case 4:
-			get = &MPD::Song::GetTrack;
-			set = &MPD::Song::SetTrack;
-			if (w == TagTypes)
+		LockStatusbar();
+		Statusbar() << "Number tracks? [" << fmtBold << 'y' << fmtBoldEnd << '/' << fmtBold << 'n' << fmtBoldEnd << "] ";
+		wFooter->Refresh();
+		int in = 0;
+		do
+		{
+			TraceMpdStatus();
+			wFooter->ReadKey(in);
+		}
+		while (in != 'y' && in != 'n');
+		UnlockStatusbar();
+		if (in == 'y')
+		{
+			MPD::SongList::iterator it = EditedSongs.begin();
+			for (unsigned i = 1; i <= EditedSongs.size(); ++i, ++it)
 			{
-				LockStatusbar();
-				Statusbar() << "Number tracks? [" << fmtBold << 'y' << fmtBoldEnd << '/' << fmtBold << 'n' << fmtBoldEnd << "] ";
-				wFooter->Refresh();
-				int in = 0;
-				do
-				{
-					TraceMpdStatus();
-					wFooter->ReadKey(in);
-				}
-				while (in != 'y' && in != 'n');
-				UnlockStatusbar();
-				if (in == 'y')
-				{
-					MPD::SongList::iterator it = EditedSongs.begin();
-					for (unsigned i = 1; i <= EditedSongs.size(); ++i, ++it)
-						(*it)->SetTrack(i);
-					ShowMessage("Tracks numbered!");
-				}
+				if (Config.tag_editor_extended_numeration)
+					(*it)->SetTrack(IntoStr(i) + "/" + IntoStr(EditedSongs.size()));
 				else
-					ShowMessage("Aborted!");
+					(*it)->SetTrack(i);
 			}
-			break;
-		case 5:
-			get = &MPD::Song::GetGenre;
-			set = &MPD::Song::SetGenre;
-			break;
-		case 6:
-			get = &MPD::Song::GetComposer;
-			set = &MPD::Song::SetComposer;
-			break;
-		case 7:
-			get = &MPD::Song::GetPerformer;
-			set = &MPD::Song::SetPerformer;
-			break;
-		case 8:
-			get = &MPD::Song::GetDisc;
-			set = &MPD::Song::SetDisc;
-			break;
-		case 9:
-			get = &MPD::Song::GetComment;
-			set = &MPD::Song::SetComment;
-			break;
-		case 10:
+			ShowMessage("Tracks numbered!");
+		}
+		else
+			ShowMessage("Aborted!");
+		return;
+	}
+	
+	if (id < 11)
+	{
+		MPD::Song::GetFunction get = Info::Tags[id].Get;
+		MPD::Song::SetFunction set = Info::Tags[id].Set;
+		if (id > 0 && w == TagTypes)
+		{
+			LockStatusbar();
+			Statusbar() << fmtBold << TagTypes->Current() << fmtBoldEnd << ": ";
+			std::string new_tag = wFooter->GetString(Tags->Current().GetTags(get));
+			UnlockStatusbar();
+			for (MPD::SongList::iterator it = EditedSongs.begin(); it != EditedSongs.end(); ++it)
+				(*it)->SetTags(set, new_tag);
+		}
+		else if (w == Tags)
+		{
+			LockStatusbar();
+			Statusbar() << fmtBold << TagTypes->Current() << fmtBoldEnd << ": ";
+			std::string new_tag = wFooter->GetString(Tags->Current().GetTags(get));
+			UnlockStatusbar();
+			if (new_tag != Tags->Current().GetTags(get))
+				Tags->Current().SetTags(set, new_tag);
+			Tags->Scroll(wDown);
+		}
+	}
+	else
+	{
+		if (id == 11) // filename related options
 		{
 			if (w == TagTypes)
 			{
@@ -603,15 +596,13 @@ void TagEditor::EnterPressed()
 					s.SetNewName(new_name + extension);
 				Tags->Scroll(wDown);
 			}
-			return;
 		}
-		case 11: // reset
+		else if (id == 12) // reset
 		{
-			Tags->Clear(0);
+			Tags->Clear();
 			ShowMessage("Changes reset");
-			return;
 		}
-		case 12: // save
+		else if (id == 13) // save
 		{
 			bool success = 1;
 			ShowMessage("Writing changes...");
@@ -620,7 +611,8 @@ void TagEditor::EnterPressed()
 				ShowMessage("Writing tags in \"%s\"...", (*it)->GetName().c_str());
 				if (!WriteTags(**it))
 				{
-					ShowMessage("Error writing tags in \"%s\"!", (*it)->GetFile().c_str());
+					static const char msg[] = "Error while writing tags in \"%s\"!";
+					ShowMessage(msg, Shorten(TO_WSTRING((*it)->GetFile()), COLS-static_strlen(msg)).c_str());
 					success = 0;
 					break;
 				}
@@ -636,47 +628,22 @@ void TagEditor::EnterPressed()
 				Mpd.UpdateDirectory(locale_to_utf_cpy(FindSharedDir(Tags)));
 			}
 			else
-				Tags->Clear(0);
-			return;
+				Tags->Clear();
 		}
-		case 13: // capitalize first letters
+		else if (id == 14) // capitalize first letters
 		{
 			ShowMessage("Processing...");
 			for (MPD::SongList::iterator it = EditedSongs.begin(); it != EditedSongs.end(); ++it)
 				CapitalizeFirstLetters(**it);
 			ShowMessage("Done!");
-			break;
 		}
-		case 14: // lower all letters
+		else if (id == 15) // lower all letters
 		{
 			ShowMessage("Processing...");
 			for (MPD::SongList::iterator it = EditedSongs.begin(); it != EditedSongs.end(); ++it)
 				LowerAllLetters(**it);
 			ShowMessage("Done!");
-			break;
 		}
-		default:
-			break;
-	}
-	
-	if (w == TagTypes && id != 0 && id != 4 && set)
-	{
-		LockStatusbar();
-		Statusbar() << fmtBold << TagTypes->Current() << fmtBoldEnd << ": ";
-		std::string new_tag = wFooter->GetString((Tags->Current().*get)());
-		UnlockStatusbar();
-		for (MPD::SongList::iterator it = EditedSongs.begin(); it != EditedSongs.end(); ++it)
-			(**it.*set)(new_tag);
-	}
-	else if (w == Tags && set)
-	{
-		LockStatusbar();
-		Statusbar() << fmtBold << TagTypes->Current() << fmtBoldEnd << ": ";
-		std::string new_tag = wFooter->GetString((Tags->Current().*get)());
-		UnlockStatusbar();
-		if (new_tag != (Tags->Current().*get)())
-			(Tags->Current().*set)(new_tag);
-		Tags->Scroll(wDown);
 	}
 }
 
@@ -695,7 +662,7 @@ void TagEditor::SpacePressed()
 	w = LeftColumn = Config.albums_in_tag_editor ? Albums : Dirs;
 	ShowMessage("Switched to %s view", Config.albums_in_tag_editor ? "albums" : "directories");
 	LeftColumn->Display();
-	Tags->Clear(0);
+	Tags->Clear();
 }
 
 void TagEditor::MouseButtonPressed(MEVENT me)
@@ -753,7 +720,7 @@ void TagEditor::MouseButtonPressed(MEVENT me)
 		}
 		else
 			Screen<Window>::MouseButtonPressed(me);
-		Tags->Clear(0);
+		Tags->Clear();
 	}
 	else if (!TagTypes->Empty() && TagTypes->hasCoords(me.x, me.y))
 	{
@@ -799,6 +766,8 @@ void TagEditor::GetSelectedSongs(MPD::SongList &v)
 {
 	std::vector<size_t> selected;
 	Tags->GetSelected(selected);
+	if (selected.empty())
+		selected.push_back(Tags->Choice());
 	for (std::vector<size_t>::const_iterator it = selected.begin(); it != selected.end(); ++it)
 		v.push_back(new MPD::Song(Tags->at(*it)));
 }
@@ -875,40 +844,28 @@ void TagEditor::PrevColumn()
 	}
 }
 
-void TagEditor::ReadTags(mpd_Song *s)
+void TagEditor::ReadTags(MPD::Song &s)
 {
-	TagLib::FileRef f(s->file);
+	TagLib::FileRef f(s.GetFile().c_str());
 	if (f.isNull())
 		return;
 	
 	TagLib::MPEG::File *mpegf = dynamic_cast<TagLib::MPEG::File *>(f.file());
 
-	s->artist = !f.tag()->artist().isEmpty() ? str_pool_get(f.tag()->artist().toCString(1)) : 0;
-	s->title = !f.tag()->title().isEmpty() ? str_pool_get(f.tag()->title().toCString(1)) : 0;
-	s->album = !f.tag()->album().isEmpty() ? str_pool_get(f.tag()->album().toCString(1)) : 0;
-	s->track = f.tag()->track() ? str_pool_get(IntoStr(f.tag()->track()).c_str()) : 0;
-	s->date = f.tag()->year() ? str_pool_get(IntoStr(f.tag()->year()).c_str()) : 0;
-	s->genre = !f.tag()->genre().isEmpty() ? str_pool_get(f.tag()->genre().toCString(1)) : 0;
+	s.SetArtist(f.tag()->artist().to8Bit(1));
+	s.SetTitle(f.tag()->title().to8Bit(1));
+	s.SetAlbum(f.tag()->album().to8Bit(1));
+	s.SetTrack(IntoStr(f.tag()->track()));
+	s.SetDate(IntoStr(f.tag()->year()));
+	s.SetGenre(f.tag()->genre().to8Bit(1));
 	if (mpegf)
 	{
-		s->composer = !mpegf->ID3v2Tag()->frameListMap()["TCOM"].isEmpty()
-		? (!mpegf->ID3v2Tag()->frameListMap()["TCOM"].front()->toString().isEmpty()
-		   ? str_pool_get(mpegf->ID3v2Tag()->frameListMap()["TCOM"].front()->toString().toCString(1))
-		   : 0)
-		: 0;
-		s->performer = !mpegf->ID3v2Tag()->frameListMap()["TOPE"].isEmpty()
-		? (!mpegf->ID3v2Tag()->frameListMap()["TOPE"].front()->toString().isEmpty()
-		   ? str_pool_get(mpegf->ID3v2Tag()->frameListMap()["TOPE"].front()->toString().toCString(1))
-		   : 0)
-		: 0;
-		s->disc = !mpegf->ID3v2Tag()->frameListMap()["TPOS"].isEmpty()
-		? (!mpegf->ID3v2Tag()->frameListMap()["TPOS"].front()->toString().isEmpty()
-		   ? str_pool_get(mpegf->ID3v2Tag()->frameListMap()["TPOS"].front()->toString().toCString(1))
-		   : 0)
-		: 0;
+		s.SetAlbumArtist(!mpegf->ID3v2Tag()->frameListMap()["TPE2"].isEmpty() ? mpegf->ID3v2Tag()->frameListMap()["TPE2"].front()->toString().to8Bit(1) : "");
+		s.SetComposer(!mpegf->ID3v2Tag()->frameListMap()["TCOM"].isEmpty() ? mpegf->ID3v2Tag()->frameListMap()["TCOM"].front()->toString().to8Bit(1) : "");
+		s.SetPerformer(!mpegf->ID3v2Tag()->frameListMap()["TOPE"].isEmpty() ? mpegf->ID3v2Tag()->frameListMap()["TOPE"].front()->toString().to8Bit(1) : "");
+		s.SetDisc(!mpegf->ID3v2Tag()->frameListMap()["TPOS"].isEmpty() ? mpegf->ID3v2Tag()->frameListMap()["TPOS"].front()->toString().to8Bit(1) : "");
 	}
-	s->comment = !f.tag()->comment().isEmpty() ? str_pool_get(f.tag()->comment().toCString(1)) : 0;
-	s->time = f.audioProperties()->length();
+	s.SetComment(f.tag()->comment().to8Bit(1));
 }
 
 namespace
@@ -929,13 +886,18 @@ void TagEditor::WriteXiphComments(const MPD::Song &s, TagLib::Ogg::XiphComment *
 	
 	tag->addField("DISCNUMBER", ToWString(s.GetDisc())); // disc
 	
+	tag->removeField("ALBUM ARTIST"); // album artist
+	GetTagList(list, s, &MPD::Song::GetAlbumArtist);
+	for (TagLib::StringList::ConstIterator it = list.begin(); it != list.end(); ++it)
+		tag->addField("ALBUM ARTIST", *it, 0);
+	
 	tag->removeField("COMPOSER"); // composer
-	GetTagList(list, s.GetComposer());
+	GetTagList(list, s, &MPD::Song::GetComposer);
 	for (TagLib::StringList::ConstIterator it = list.begin(); it != list.end(); ++it)
 		tag->addField("COMPOSER", *it, 0);
 	
 	tag->removeField("PERFORMER"); // performer
-	GetTagList(list, s.GetPerformer());
+	GetTagList(list, s, &MPD::Song::GetPerformer);
 	for (TagLib::StringList::ConstIterator it = list.begin(); it != list.end(); ++it)
 		tag->addField("PERFORMER", *it, 0);
 }
@@ -971,10 +933,13 @@ bool TagEditor::WriteTags(MPD::Song &s)
 			WriteID3v2("TCON", tag, ToWString(s.GetGenre()));  // genre
 			WriteID3v2("TPOS", tag, ToWString(s.GetDisc()));   // disc
 			
-			GetTagList(list, s.GetComposer());
+			GetTagList(list, s, &MPD::Song::GetAlbumArtist);
+			WriteID3v2("TPE2", tag, list); // album artist
+			
+			GetTagList(list, s, &MPD::Song::GetComposer);
 			WriteID3v2("TCOM", tag, list); // composer
 			
-			GetTagList(list, s.GetPerformer());
+			GetTagList(list, s, &MPD::Song::GetPerformer);
 			// in >=mpd-0.16 treating TOPE frame as performer tag
 			// was dropped in favor of TPE3/TPE4 frames, so we have
 			// to write frame accurate to used mpd version
@@ -1000,7 +965,7 @@ bool TagEditor::WriteTags(MPD::Song &s)
 			locale_to_utf(new_name);
 			if (rename(path_to_file.c_str(), new_name.c_str()) == 0 && !file_is_from_db)
 			{
-				if (myOldScreen == myPlaylist)
+				if (Global::myOldScreen == myPlaylist)
 				{
 					// if we rename local file, it won't get updated
 					// so just remove it from playlist and add again
@@ -1016,7 +981,7 @@ bool TagEditor::WriteTags(MPD::Song &s)
 					Mpd.CommitCommandsList();
 				}
 				else // only myBrowser->Main()
-					s.SetFile(new_name);
+					myBrowser->GetDirectory(myBrowser->CurrentDir());
 			}
 		}
 		return true;
@@ -1042,106 +1007,43 @@ std::string TagEditor::CapitalizeFirstLetters(const std::string &s)
 
 void TagEditor::CapitalizeFirstLetters(MPD::Song &s)
 {
-	s.SetTitle(CapitalizeFirstLetters(s.GetTitle()));
-	s.SetArtist(CapitalizeFirstLetters(s.GetArtist()));
-	s.SetAlbum(CapitalizeFirstLetters(s.GetAlbum()));
-	s.SetGenre(CapitalizeFirstLetters(s.GetGenre()));
-	s.SetComposer(CapitalizeFirstLetters(s.GetComposer()));
-	s.SetPerformer(CapitalizeFirstLetters(s.GetPerformer()));
-	s.SetDisc(CapitalizeFirstLetters(s.GetDisc()));
-	s.SetComment(CapitalizeFirstLetters(s.GetComment()));
+	for (const Info::Metadata *m = Info::Tags; m->Name; ++m)
+	{
+		unsigned i = 0;
+		for (std::string tag; !(tag = (s.*m->Get)(i)).empty(); ++i)
+			(s.*m->Set)(CapitalizeFirstLetters(tag), i);
+	}
 }
 
 void TagEditor::LowerAllLetters(MPD::Song &s)
 {
-	std::string conv = s.GetTitle();
-	ToLower(conv);
-	s.SetTitle(conv);
-	
-	conv = s.GetArtist();
-	ToLower(conv);
-	s.SetArtist(conv);
-	
-	conv = s.GetAlbum();
-	ToLower(conv);
-	s.SetAlbum(conv);
-	
-	conv = s.GetGenre();
-	ToLower(conv);
-	s.SetGenre(conv);
-	
-	conv = s.GetComposer();
-	ToLower(conv);
-	s.SetComposer(conv);
-	
-	conv = s.GetPerformer();
-	ToLower(conv);
-	s.SetPerformer(conv);
-	
-	conv = s.GetDisc();
-	ToLower(conv);
-	s.SetDisc(conv);
-	
-	conv = s.GetComment();
-	ToLower(conv);
-	s.SetComment(conv);
+	for (const Info::Metadata *m = Info::Tags; m->Name; ++m)
+	{
+		unsigned i = 0;
+		for (std::string tag; !(tag = (s.*m->Get)(i)).empty(); ++i)
+		{
+			ToLower(tag);
+			(s.*m->Set)(tag, i);
+		}
+	}
 }
 
-void TagEditor::GetTagList(TagLib::StringList &list, const std::string &s)
+void TagEditor::GetTagList(TagLib::StringList &list, const MPD::Song &s, MPD::Song::GetFunction f)
 {
 	list.clear();
-	for (size_t i = 0; i != std::string::npos; i = s.find(",", i))
-	{
-		if (i)
-			++i;
-		while (s[i] == ' ')
-			++i;
-		size_t j = s.find(",", i);
-		list.append(ToWString(s.substr(i, j-i)));
-	}
+	unsigned pos = 0;
+	for (std::string value; !(value = (s.*f)(pos)).empty(); ++pos)
+		list.append(ToWString(value));
 }
 
 std::string TagEditor::TagToString(const MPD::Song &s, void *data)
 {
 	std::string result;
-	switch (static_cast<Menu<std::string> *>(data)->Choice())
-	{
-		case 0:
-			result = s.GetTitle();
-			break;
-		case 1:
-			result = s.GetArtist();
-			break;
-		case 2:
-			result = s.GetAlbum();
-			break;
-		case 3:
-			result = s.GetDate();
-			break;
-		case 4:
-			result = s.GetTrack();
-			break;
-		case 5:
-			result = s.GetGenre();
-			break;
-		case 6:
-			result = s.GetComposer();
-			break;
-		case 7:
-			result = s.GetPerformer();
-			break;
-		case 8:
-			result = s.GetDisc();
-			break;
-		case 9:
-			result = s.GetComment();
-			break;
-		case 11:
-			result = s.GetNewName().empty() ? s.GetName() : s.GetName() + " -> " + s.GetNewName();
-			break;
-		default:
-			break;
-	}
+	size_t i = static_cast<Menu<std::string> *>(data)->Choice();
+	if (i < 11)
+		result = (s.*Info::Tags[i].Get)(0);
+	else if (i == 12)
+		result = s.GetNewName().empty() ? s.GetName() : s.GetName() + " -> " + s.GetNewName();
 	return result.empty() ? Config.empty_tag : result;
 }
 
@@ -1179,6 +1081,8 @@ MPD::Song::SetFunction TagEditor::IntoSetFunction(char c)
 	{
 		case 'a':
 			return &MPD::Song::SetArtist;
+		case 'A':
+			return &MPD::Song::SetAlbumArtist;
 		case 't':
 			return &MPD::Song::SetTitle;
 		case 'b':
@@ -1256,7 +1160,7 @@ std::string TagEditor::ParseFilename(MPD::Song &s, std::string mask, bool previe
 		{
 			MPD::Song::SetFunction set = IntoSetFunction(it->first);
 			if (set)
-				(s.*set)(it->second);
+				s.SetTags(set, it->second);
 		}
 		else
 			result << "%" << it->first << ": " << it->second << "\n";
