@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008-2009 by Andrzej Rybczak                            *
+ *   Copyright (C) 2008-2010 by Andrzej Rybczak                            *
  *   electricityispower@gmail.com                                          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -53,6 +53,7 @@ class TagEditor : public Screen<Window>
 		virtual void EnterPressed();
 		virtual void SpacePressed();
 		virtual void MouseButtonPressed(MEVENT);
+		virtual bool isTabbable() { return true; }
 		
 		virtual MPD::Song *CurrentSong();
 		
@@ -76,7 +77,7 @@ class TagEditor : public Screen<Window>
 		/// NOTICE: this string is always in utf8, no need to convert it
 		const std::string &CurrentDir() { return itsBrowsedDir; }
 		
-		static void ReadTags(mpd_Song *);
+		static void ReadTags(MPD::Song &);
 		static bool WriteTags(MPD::Song &);
 		
 	protected:
@@ -96,7 +97,7 @@ class TagEditor : public Screen<Window>
 		static std::string CapitalizeFirstLetters(const std::string &);
 		static void CapitalizeFirstLetters(MPD::Song &);
 		static void LowerAllLetters(MPD::Song &);
-		static void GetTagList(TagLib::StringList &, const std::string &);
+		static void GetTagList(TagLib::StringList &, const MPD::Song &, MPD::Song::GetFunction);
 		static void WriteXiphComments(const MPD::Song &, TagLib::Ogg::XiphComment *);
 		
 		static void GetPatternList();
