@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2008-2009 by Andrzej Rybczak                            *
+ *   Copyright (C) 2008-2010 by Andrzej Rybczak                            *
  *   electricityispower@gmail.com                                          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -21,6 +21,12 @@
 #include <cstring>
 #include <cstdlib>
 
+#ifdef WIN32
+# include <winsock.h>
+#else
+# include <sys/select.h>
+#endif
+
 #include "error.h"
 #include "window.h"
 
@@ -149,7 +155,6 @@ Window::~Window()
 {
 	delwin(itsWindow);
 	delwin(itsWinBorder);
-	delete itsHistory;
 }
 
 void Window::SetColor(Color fg, Color bg)
@@ -225,7 +230,7 @@ void Window::SetTitle(const std::string &new_title)
 void Window::CreateHistory()
 {
 	if (!itsHistory)
-		itsHistory = new std::deque<std::wstring>;
+		itsHistory = new std::list<std::wstring>;
 }
 
 void Window::DeleteHistory()
@@ -312,11 +317,9 @@ void Window::Refresh()
 	prefresh(itsWindow, 0, 0, itsStartY, itsStartX, itsStartY+itsHeight-1, itsStartX+itsWidth-1);
 }
 
-void Window::Clear(bool refresh)
+void Window::Clear()
 {
 	werase(itsWindow);
-	if (refresh)
-		Window::Refresh();
 }
 
 void Window::Hide(char ch) const
@@ -352,9 +355,63 @@ void Window::SetTimeout(int timeout)
 	wtimeout(itsWindow, timeout);
 }
 
+void Window::AddFDCallback(int fd, void (*callback)())
+{
+	itsFDs.push_back(std::make_pair(fd, callback));
+}
+
+void Window::ClearFDCallbacksList()
+{
+	itsFDs.clear();
+}
+
+bool Window::FDCallbacksListEmpty() const
+{
+	return itsFDs.empty();
+}
+
 void Window::ReadKey(int &read_key) const
 {
+	// in pdcurses polling stdin doesn't work, so we can't poll
+	// both stdin and other file descriptors in one select. the
+	// workaround is to set the timeout of select to 0, poll
+	// other file descriptors and then wait for stdin input with
+	// the given timeout. unfortunately, this results in delays
+	// since ncmpcpp doesn't see that data arrived while waiting
+	// for input from stdin, but it seems there is no better option.
+	
+	fd_set fdset;
+	FD_ZERO(&fdset);
+#	if !defined(USE_PDCURSES)
+	FD_SET(STDIN_FILENO, &fdset);
+	timeval timeout = { itsWindowTimeout/1000, (itsWindowTimeout%1000)*1000 };
+#	else
+	timeval timeout = { 0, 0 };
+#	endif
+	
+	int fd_max = STDIN_FILENO;
+	for (FDCallbacks::const_iterator it = itsFDs.begin(); it != itsFDs.end(); ++it)
+	{
+		if (it->first > fd_max)
+			fd_max = it->first;
+		FD_SET(it->first, &fdset);
+	}
+	
+	if (select(fd_max+1, &fdset, 0, 0, itsWindowTimeout < 0 ? 0 : &timeout) > 0)
+	{
+#		if !defined(USE_PDCURSES)
+		read_key = FD_ISSET(STDIN_FILENO, &fdset) ? wgetch(itsWindow) : ERR;
+#		endif // !USE_PDCURSES
+		for (FDCallbacks::const_iterator it = itsFDs.begin(); it != itsFDs.end(); ++it)
+			if (FD_ISSET(it->first, &fdset))
+				it->second();
+	}
+#	if !defined(USE_PDCURSES)
+	else
+		read_key = ERR;
+#	else
 	read_key = wgetch(itsWindow);
+#	endif
 }
 
 void Window::ReadKey() const
@@ -378,7 +435,7 @@ std::string Window::GetString(const std::string &base, size_t length, size_t wid
 	
 	std::wstring wbase = ToWString(base);
 	std::wstring *tmp = &wbase;
-	size_t history_offset = itsHistory && !encrypted ? itsHistory->size() : -1;
+	std::list<std::wstring>::iterator history_it = itsHistory->end();
 	
 	std::string tmp_in;
 	wchar_t wc_in;
@@ -437,7 +494,7 @@ std::string Window::GetString(const std::string &base, size_t length, size_t wid
 		
 		wmove(itsWindow, y, x);
 		prefresh(itsWindow, 0, 0, itsStartY, itsStartX, itsStartY+itsHeight-1, itsStartX+itsWidth-1);
-		input = wgetch(itsWindow);
+		ReadKey(input);
 		
 		// these key codes are special and should be ignored
 		if ((input < 10 || (input > 10 && input != 21 && input < 32))
@@ -454,28 +511,22 @@ std::string Window::GetString(const std::string &base, size_t length, size_t wid
 			case KEY_MOUSE:
 				break;
 			case KEY_UP:
-				if (itsHistory && !encrypted && history_offset > 0)
+				if (itsHistory && !encrypted && history_it != itsHistory->begin())
 				{
-					do
-						tmp = &(*itsHistory)[--history_offset];
-					while (tmp->empty() && history_offset);
+					while (--history_it != itsHistory->begin())
+						if (!history_it->empty())
+							break;
+					tmp = &*history_it;
 					gotoend = 1;
 				}
 				break;
 			case KEY_DOWN:
-				if (itsHistory && !encrypted && itsHistory->size() > 0)
+				if (itsHistory && !encrypted && history_it != itsHistory->end())
 				{
-					if (history_offset < itsHistory->size()-1)
-					{
-						do
-							tmp = &(*itsHistory)[++history_offset];
-						while (tmp->empty() && history_offset < itsHistory->size()-1);
-					}
-					else if (history_offset == itsHistory->size()-1)
-					{
-						tmp = &wbase;
-						history_offset++;
-					}
+					while (++history_it != itsHistory->end())
+						if (!history_it->empty())
+							break;
+					tmp = &(history_it == itsHistory->end() ? wbase : *history_it);
 					gotoend = 1;
 				}
 				break;
@@ -484,7 +535,7 @@ std::string Window::GetString(const std::string &base, size_t length, size_t wid
 				if (x < maxx)
 				{
 					real_x++;
-					x += wcwidth((*tmp)[beginning+real_x-minx-1]);
+					x += std::max(wcwidth((*tmp)[beginning+real_x-minx-1]), 1);
 				}
 				else if (beginning < maxbeginning)
 					beginning++;
@@ -500,7 +551,7 @@ std::string Window::GetString(const std::string &base, size_t length, size_t wid
 				if (x > minx)
 				{
 					real_x--;
-					x -= wcwidth((*tmp)[beginning+real_x-minx]);
+					x -= std::max(wcwidth((*tmp)[beginning+real_x-minx]), 1);
 				}
 				else if (beginning > 0)
 					beginning--;
@@ -580,16 +631,14 @@ std::string Window::GetString(const std::string &base, size_t length, size_t wid
 	
 	if (itsHistory && !encrypted)
 	{
-		size_t old_size = itsHistory->size();
-		if (!tmp->empty() && (itsHistory->empty() || itsHistory->back() != *tmp))
-			itsHistory->push_back(*tmp);
-		if (old_size > 1 && history_offset < old_size)
+		if (history_it != itsHistory->end())
 		{
-			std::deque<std::wstring>::iterator it = itsHistory->begin()+history_offset;
-			wbase = *it;
-			tmp = &wbase;
-			itsHistory->erase(it);
+			itsHistory->push_back(*history_it);
+			tmp = &itsHistory->back();
+			itsHistory->erase(history_it);
 		}
+		else
+			itsHistory->push_back(*tmp);
 	}
 	
 	return ToString(*tmp);
@@ -688,6 +737,11 @@ Border Window::GetBorder() const
 	return itsBorder;
 }
 
+int Window::GetTimeout() const
+{
+	return itsWindowTimeout;
+}
+
 void Window::Scroll(Where where)
 {
 	idlok(itsWindow, 1);
@@ -886,9 +940,7 @@ std::wstring ToWString(const std::string &s)
 
 size_t Window::Length(const std::wstring &ws)
 {
-	size_t length = 0;
-	for (std::wstring::const_iterator it = ws.begin(); it != ws.end(); ++it)
-		length += wcwidth(*it);
-	return length;
+	int len = wcswidth(ws.c_str(), -1);
+	return len < 0 ? ws.length() : len;
 }
 
